import constants as c_
from common_ import print_, RaiseNotImplementedError
from snd_holder import snd_holder


class char:
    charname = "UNNAMED CHARACTER"
    _SH_FIRE = snd_holder(c_._SG_FIRE, 0)
    _SH_NOAMMO = snd_holder(c_._SG_FIRE, 1)
    def __init__(self, stor, player, spawn_vect, spawn_data = None):
        """
        spawn_vect = (x, y, dx, dy),
        coords must be given assuming that:
            screen.topleft     == (-1, -1) and
            screen.bottomright == ( 1,  1)

        spawn_data is any object, specific to a character
        """
        self.stor = stor
        self.player = player
        self.player_id = player.player_id
        self.string = "%s of %s" % (self.charname, player.string)
        print_( "Creating %s" % self.string)
        
        assert(c_._OG_PLAYER2 == c_._OG_PLAYER1 + 1)
        self.obj_group = c_._OG_PLAYER1 - 1 + self.player_id


        #sounds
        self.init_sounds()


        #body
        self.balls, self.meats = [], []
        self.total_attached = None #create_body must fill these values

        assert(len(spawn_vect) == 4)
        ph = self.stor.physics
        spawn_vect = map(lambda xy, rxy: xy * rxy,   spawn_vect, (ph.rx, ph.ry) * 2)
        self.create_body(spawn_vect, spawn_data)

        #permanent animations
        self.add_initial_anim()

        #params
        self.is_damagable = True
        self.target_ball = None
        
        self.set_initial_params()

    def create_body(self, spawn_vect, spawn_data):
        """
        spawn vect is a (x, y, dx, dy) tuple, passed in world coords
        spawn_data is any object, specific to a character
        """
        RaiseNotImplementedError()

    def add_initial_anim(self):
        RaiseNotImplementedError()

    def set_initial_params(self):
        RaiseNotImplementedError()

    def render_step0(self, stor, visible):
        RaiseNotImplementedError()

    def render_step1(self, stor, visible):
        RaiseNotImplementedError()

    def get_cursable_balls(self):
        RaiseNotImplementedError()
        

    def init_sounds(self):
        self.now_sound = None
        self.sh_fire = self._SH_FIRE
        self.sh_noammo = self._SH_NOAMMO

    def kill_meat(self, a, b):
        "remove link between balls a and b"
        fnd = False
        for m in self.meats:
            for (x, y) in ((a, b), (b, a)):
                if (m.a is x) and (m.b is y):
                    fnd = True
                    self.meats.remove(m)
                    break
                
        if not fnd:
            raise "attempted to delete a non-existing meat from %s" % self.string



if __name__ == "__main__":
    def RaiseNotImplementedError():
        print "NotImplementedError is disabled for unit testing"

    import threading, sys

    class dummy_player:
        player_id = 1
        string = "DUMMY PLAYER"
        pass

    class dummy_physics:
        rx, ry = 3, 3
        pass
        
    class dummy_stor:
        print_lock = threading.Lock()
        physics = dummy_physics()
        pass

    from common_ import set_storage
    player = dummy_player()
    stor = dummy_stor()
    set_storage(stor)
    spawn_rect = (0, 0, -0.25, 0)    
    t = char(stor, player, spawn_rect)

    try:
        t.kill_meat(None, None)
    except:
        print "raised:", sys.exc_info()[0]
        
    
