import math
from my_maths import angle_a_to_b_rad, period_to_range, dist_a_to_b
from comm_interface import comm_interface

def time_scale_changed(new_ts):
    ci_ai_spider._timescale = new_ts

def difficulty_changed(new_diffic):
    c = ci_ai_spider
    def calc((v0, v1)):
        return v0 + (v1 - v0) * new_diffic / 4.0
        

    c.critical_hp = calc(c.def_critical_hp)
    c.critical_hp_diff = calc(c.def_critical_hp_diff)
    c.speed = calc(c.def_speed)
    
class ci_ai_spider(comm_interface):
    _timescale = 1.0
    def_critical_hp = (10, 50)
    def_critical_hp_diff = (50, 1)
    def_speed = (0.005, 0.035)
    critical_hp = critical_hp_diff = speed = None

    see_walls_radius = 3 #multiplied by head's radius
    see_walls_angle = math.pi / 3
    see_walls = see_walls_radius, see_walls_angle

    def __init__(self, daddy = None, opponent = None):
        comm_interface.__init__(self, daddy, opponent)
        self.my_info = self.daddy.info
        self.his_info = self.opponent.info
        self.my_head = self.daddy.char.head
        self.his_head = self.opponent.char.head
        self.spidies = self.daddy.char.spidies
        self.find_best_spidy = self.daddy.char.find_attached_spidy_closest_to_target
        
    
    def get_events_(self, stor, o_data):

        me = self.my_info
        my_head = self.my_head
        he = self.his_info

        #determine target
        target = None
        
        i_want_food = ((me.health <= self.critical_hp) or
                       (he.health - me.health >= self.critical_hp_diff))
    
        if i_want_food:
            #try find a bounty
            best_bounty, best_estim = None, None
            for b in stor.bounties:
                if b.eatable and (b.type in [1, 2]):
                    #maybe can't get it till it fades?
                    dist = dist_a_to_b(my_head.crd, b.crd)
                    if ((b.lifetime <= 1.0) and (dist > my_head.r * 10.0)):
                        continue
                    
                    #evaluate
                    estim = b.type / dist
                    if ((best_estim is None) or (best_estim < estim)):
                        best_estim = estim
                        best_bounty = b
                        
            target = best_bounty #maybe None

        if target is None:
            target = self.his_head


        #calc move towards target
        dx, dy = target.crd[0] - my_head.crd[0], target.crd[1] - my_head.crd[1]
        k = self.speed / dist_a_to_b((dx, dy), (0, 0))
        move = [dx * k, dy * k]

        #dodge walls
        see_r, see_a = self.see_walls
        see_r *= my_head.r
        for wall in stor.walls:
            wdx, wdy = wall.crd[0] - my_head.crd[0], wall.crd[1] - my_head.crd[1]
            if ((abs(wdx) <= see_r) and (abs(wdy) <= see_r)):
                d = dist_a_to_b((wdx, wdy), (0, 0))
                if d <= see_r:
                    ang0 = angle_a_to_b_rad(my_head.crd, target.crd)
                    ang1 = angle_a_to_b_rad(my_head.crd, wall.crd)
                    da = period_to_range(ang1 - ang0, -math.pi, math.pi)
                    if abs(da) <= see_a / 2:
                        #avoid wall
                        move_ = move[:]
                        
                        move[0] = move_[1]
                        move[1] = -move_[0]

                        break                        

        #movement done. fire?
        fire = 0
        closest, closest_dist = self.find_best_spidy()
        if closest != None:
            dist = dist_a_to_b(my_head.crd, self.daddy.char.target_ball.crd)
            if dist >= closest_dist:
                fire = 1

        
        #done
        o_data.extend([move, 0, fire, 0])
