from math import *
from pygame import Rect

def calc_prc_area(area, prc_area):
    sr = area
    x0, y0 = sr.topleft
    w, h = sr.size
    return Rect(x0 + w * prc_area[0] / 100, y0 + h * prc_area[1] / 100,
                w * prc_area[2] / 100, h * prc_area[3] / 100)
def czero(x):
    if x > 1e-8:
        return 1
    elif x < -1e-8:
        return -1
    else:
        return 0

def period_to_range(x, a, b):
    assert(b >= a)
    d = b - a
    while czero(x - b) >= 0: x -= d
    while czero(x - a) == -1: x += d
    return x
    


def angle_a_to_b(a, b):
    x1, y1 = a
    x2, y2 = b
    return atan2(-(y2 - y1), x2 - x1) / pi * 180.0

def angle_a_to_b_rad(a, b):
    x1, y1 = a
    x2, y2 = b
    return atan2(-(y2 - y1), x2 - x1)

def my_sqrt(v):
    if v < 1e-10:
        return 0
    else:
        return sqrt(v)

def dist_a_to_b(a, b):
    dx, dy = a[0] - b[0], a[1] - b[1]
    return my_sqrt(dx * dx + dy * dy)

class point:
    def __init__(self, x, y):
        self.x = float(x)
        self.y = float(y)

    def len(self):
        return sqrt(self.x * self.x + self.y * self.y)
    
    def normalize(self):
        d = self.len()
        self.x /= d
        self.y /= d
        return self

    def normalized(self):
        d = self.len()
        return point(self.x / d, self.y / d)
                     
    def mult_by_k(self, k):
        self.x *= float(k)
        self.y *= float(k)
        return self

    def dist_to_b(self, b):
        return point(b.x - self.x, b.y - self.y).len()

    def add_b(self, b):
        self.x += b.x
        self.y += b.y
        return self
        

class vect(list):
    def len(self):
        x, y = (float(self[0]), float(self[1]))
        return sqrt(x * x + y * y)

    def dist_to_b(self, b):
        "distance between points"
        return vect([self[0] - b[0], self[1] - b[1]]).len()

    def mult_by_x(self, x):
        return vect([self[0] * x, self[1] * x])

    def add_b(self, b):
        return vect([self[0] + b[0], self[1] + b[1]])

    def normalize(self):
        koef = self.len() or 1.0
        self[0] = float(self[0]) / koef
        self[1] = float(self[1]) / koef

    def normalized(self):
        b = vect(self);
        b.normalize()
        return b

    def cosine(self, b):
        "cosine between vectors"
        la, lb = (self.len() or 1.0, b.len() or 1.0)
        xa, ya = (self[0] / la, self[1] / la)
        xb, yb = (b[0] / lb, b[1] / lb)
        return xa * xb + ya * yb

    def sine(self, b):
        "sine between vectors"
        la, lb = (self.len() or 1.0, b.len() or 1.0)
        xa, ya = (self[0] / la, self[1] / la)
        xb, yb = (b[0] / lb, b[1] / lb)
        return xa * yb - ya * xb

    def rot_at_b(self, b):
        """returns self rotated by angle b counterclockwise
                b in radians"""
        la = self.len() or 1.0
#        print la
        xa, ya = (self[0] / la, self[1] / la)
        xb, yb = cos(b), sin(b)
        sq2 = 1.0 / sqrt(2.0)
        return vect([xa * xb - ya * yb, xa * yb + ya * xb])

    def decomposed_by_b(self, b):
        return decompose_vector(self, b)


def normalized(v):
    """accepts (x, y), returns (x', y') - normalized (x, y)
    x' * x' + y' * y' == 1"""

    x, y = v
    leng = sqrt(x * x + y * y)
    return (x / leng, y / leng)


def decompose_vector(v, v1):
    """
    accepts (x, y), (x1, y1)

    let's assume that (x2, y2) is perpendicular to (x1, y1); say, (y1, x1)

    decomposes (x, y), representing it as sum: k1 v1 + k2 v2

    returns ((ax1, ay1), (ax2, ay2)) such that

            1. (ax1, ay1) is collinear to (x1, y1)
            2. (ax2, ay2) is collinear to (x2, y2)
            3. (ax1, ay1) + (ax2, ay2) = (x, y)"""

    
    x, y = v
    lxy = v.len()
    v1.normalize()
    x1, y1 = v1
    x2, y2 = v1.rot_at_b(pi / 2.0)

#    print (x2, y2)
    
    newl1 = lxy * v.cosine(v1)
    newl2 = - lxy * v.sine(v1)
    
    return (vect([x1 * newl1, y1 * newl1]), vect([x2 * newl2, y2 * newl2]))


if __name__ == "__main__":
#    print decompose_vector((2, 3), (4, 5))
#    print normalized((2, 2))
    a = vect([2, 2])
    a.normalize()
    print a
    b = vect([0, 1])
    print b
    print b.rot_at_b(pi / 2.0)
    print b.rot_at_b(pi / 2.0).rot_at_b(pi / 2.0)
    print decompose_vector(vect([0, 1]), vect([1, 1]))
    print decompose_vector(vect([5, 5]), vect([3, 1]))

    print "\npart2"
    a = vect([0, 1])
    b = vect([2, 2])
    print a.decomposed_by_b(b)

    print "\npart3"
    a = vect([0, 1])
    print a.len() or 1.0
    a = vect([0, 0])
    print a.len() or 1.0
