from ball import ball
import time
import constants as c_
from img_holder import img_holder
from common_ import print_
from random import randint
import bounty_effect
import threading
from player_info import player_info
from snd_holder import snd_holder
from snd_list import snd_list

IG_BNT = (c_._IG_BOUNTY_BAD, c_._IG_BOUNTY_HP, c_._IG_BOUNTY_HP2)
AFF_BNT = ('CURSE', 'HP10', 'HP25')
LIFETIME = (4, 6, 3)
##LIFETIME = map(lambda x: x * 2, LIFETIME)
##LIFETIME = (1, 1)
n_types = len(IG_BNT)

(PHASE_INIT
,PHASE_THR_CREATION
,PHASE_RELEASE
,PHASE_EATEN
,PHASE_THR_FADING
,PHASE_IDLE
) = range(6)
#note that we're only intersted in all threaded phases (THR_) and
#those non-threaded preceded by threaded (to stop those)
#
#for the rest, we only do an initial lock-n-release,
#not to stop previous threaded phases,
#but to make sure no future phase is running

def handle_event_NEW_BOUNTY(i_gm, (i_daddy_cloud_id, bnt_type)):

    #find idle bounty and replace, or create a new one
    bnt = i_gm.stor.bounties
    new_id = len(bnt)
    old_b = None

    for b in bnt:
        if b.idle:
            new_id = b.id
            old_b = b
            break

    new_b = bounty(c_._OG_BOUNTIES, i_gm.stor, 
                   daddy_cloud_id = i_daddy_cloud_id,
                   id = new_id, gm = i_gm, type = bnt_type)

    if not old_b:
        bnt.append(new_b)
    else:
        assert(new_id == bnt.index(old_b))
        bnt[new_id] = new_b
    
def gen_rand_type():
    return randint(0, n_types - 1)

class bounty(ball):
    def __init__(self, obj_group, stor,
                 img_gr = c_._IG_BOUNTY_321, img_id = 2,
                 friction = 0.001, solidity = 1.0,
                 string = "BOUNTY",
                 def_size = c_._WH_BOUNTY,
                 collidable = False,
                 daddy_cloud_id = None,
                 id = None,
                 gm = None,
                 type = None,
                 autorun = True):
        self.daddy = stor.clouds[daddy_cloud_id]
        self.phase = PHASE_INIT
        self.phase_lock = threading.Lock()

        x, y = self.daddy.crd
        ball.__init__(self, obj_group, stor, x, y,
                      img_gr, img_id,
                      friction, solidity,
                      string, def_size,
                      collidable)
        
        self.snapped = True
        self.eatable = False
        self.id = id
        self.gm = gm
        self.idle = False
        self.alpha = 255
        self.walls_below = [True] * len(self.stor.walls)
        self.n_walls_below = len(self.stor.walls)
        self.type = type
        self.lifetime = None
        self.died = 0

        self.sl_effect = snd_list(self.stor.sndf, (c_._SG_BOUNTY, 0, 3))
        self.sh_created = snd_holder(c_._SG_BOUNTY, 3)
        self.sh_tick = snd_holder(c_._SG_TIME, 0)
        self.sh_lost = snd_holder(c_._SG_BOUNTY, 4)
        self.now_sound = None

        if autorun:
            gm.start_thread(self.cycle_create, ())

    def lock_new_phase(self, new_phase):
        """False if we are in this or later phase already
        if True, locks self.phase_lock"""

        toret = True
        self.phase_lock.acquire()
        if self.phase >= new_phase:
            print_ ("Error when setting phase to %d, because already was in phase %d" % (new_phase, self.phase))
            toret = False
            self.phase_lock.release()
        else:
            self.phase = new_phase
        return toret
    
    def lock_if_still_in_phase(self, cur_phase):
        """False if moved to future phase already;
        assert if cur_phase > self.phase, because this means
        that we can't be "still_in_phase" since
        we have never entered it

        required to immediately exit some thread if it's phase
        has ended before it wanted (i.e. next phase has started)

        if True, locks self.phase_lock

        really needed only for client-side
        for server, makes sure phase-order logic is correct"""
        
        toret = True
        self.phase_lock.acquire()
        assert(self.phase >= cur_phase)
        if self.phase > cur_phase:
            toret = False
            self.phase_lock.release()
        return toret
            
    def need_emergently_exit_phase(self, phase):
        """if False, locks self.phase_lock"""
        if (self.gm.threads_must_stop or
            (not self.lock_if_still_in_phase(phase))):
            self.gm.some_thread_stopped()
            return True
        else:
            return False



    def render(self, stor, visible):
##        ball.render(self, stor, visible, self.get_debug)

        if self.idle:
            return
        
        ball.render(self, stor, visible, self.get_alpha_pic)
        if (self.snapped and (self.n_walls_below > 0)) or self.died:
            ball.render(self, stor, visible, self.get_red_pic)

    def get_debug(self):
        ih = img_holder(c_._IG_BOUNTY_321, 3)
        return self.stor.imgf.get_image(ih)

    def get_alpha_pic(self):
        toret = self.imgf_get_image_alpha(self.ih_main, alpha = self.alpha)
##        print toret.img[0].get_colorkey()
        return toret
        

    def get_red_pic(self):
        ih = img_holder(c_._IG_BOUNTY_321, 3)
        if self.died:
            if self.died == 2: ih.img_id = 4 #yellow
            return self.imgf_get_image_alpha(ih, alpha = self.alpha)
        else:
            return self.imgf_get_image(ih)


    def cycle_create(self):
        assert(self.lock_new_phase(PHASE_THR_CREATION))

        delta = 0.5
        self.ih_main.img_gr = c_._IG_BOUNTY_321
        self.phase_lock.release()

        for i in [2, 2, 1, 1, 0]:
            if self.need_emergently_exit_phase(PHASE_THR_CREATION): return
            self.ih_main.img_id = i
            self.phase_lock.release()
            self.now_sound = self.stor.sndf.get_sound(self.sh_tick)


            time.sleep(delta)

        self.now_sound = self.stor.sndf.get_sound(self.sh_tick)

        if self.need_emergently_exit_phase(PHASE_THR_CREATION): return
        self.ih_main = img_holder(IG_BNT[self.type],
                                  self.stor.map.data["bounty_type%d" % self.type])
        self.phase_lock.release()

        self.now_sound = self.stor.sndf.get_sound(self.sh_created)

        delta = 0.1
        for i in range(50, 255, 50):
            if self.need_emergently_exit_phase(PHASE_THR_CREATION): return
            self.alpha = i
            self.phase_lock.release()

            time.sleep(delta)



        if not self.gm.client_side():
            #at clientside we will get the RELEASE_BOUNTY event via net
            delta = 0.1
            while self.n_walls_below > 0:
                if self.need_emergently_exit_phase(PHASE_THR_CREATION): return
                #do nothing; n_walls_below are changed in
                #get_hit_by_b and get_unhit_by_b (called from physics)
                self.phase_lock.release()

                time.sleep(delta)
 
            self.gm.new_event(c_._EVENT_RELEASE_BOUNTY, (self.id, 0))

        self.gm.some_thread_stopped()

    def release_from_daddy(self):
        assert(self.lock_new_phase(PHASE_RELEASE))
        self.phase_lock.release()
        #if there was no IDLE phase, there would be
        #no need in further phases control, because none of them would
        #be preceded by a threaded-one (so there's no danger that
        #previous, threaded phase still runs while we're in the next one

        self.collidable = True
        self.eatable = True
        self.snapped = False
        self.speed = self.daddy.speed[:]
        self.lifetime = LIFETIME[self.type]

        player_below = self.daddy.belong
        if player_below: 
            self.get_eaten_by(player_below.char)

    def make_step(self, dt):
        if self.lifetime is None:
            return

        prev_lifetime = self.lifetime
        self.lifetime -= dt
        if self.lifetime <= 0:
            self.died = 1
            self.now_sound = self.stor.sndf.get_sound(self.sh_lost)
            self.get_eaten_by(None)
        else:
            #blink
            t = int(self.lifetime * 8)
            old_t = int(prev_lifetime * 8)

            if t != old_t: # to sound properly
                if t <= 20-1:
                    self.died = ((t % 4 == 3)
                                 and ((t / 4 < 2)
                                      and 1
                                      or 2)
                                 or 0)
                    if self.died:
                        self.now_sound = self.stor.sndf.get_sound(self.sh_tick)
        
        
        
    def snap(self):
        self.crd = self.daddy.crd
 
    def get_hit_by_b(self, b, stop_check):
##        print "in"
##        if stop_check:
##            print "from second cycle"
        if not self.snapped:
##            print "hitting?"
            if self.collidable and b.collidable and b.recognizable_by_clouds():
##                print "hit!"
                ball.get_hit_by_b(self, b, stop_check)
            return
        
        if b.obj_gr == c_._OG_WALLS:
            w_id = self.stor.walls.index(b)
            wb = self.walls_below
            if not wb[w_id]:
                wb[w_id] = True
                self.n_walls_below += 1

    def damages_b(self, b):
##        print_( "checking damage from bounty" )
##        print_( "eatable: %s" % self.eatable)
            
        maybe = (b.obj_gr in c_._OGs_PLAYERS) and b.attached and self.eatable
        if maybe:
            also = True
            if self.type == 0:
                also = also and len(b.char.get_cursable_balls()) > 0
            if self.type in [1, 2]:
                also = also and b.player.info.health < player_info.def_start_health
            maybe = also

        return maybe

    def damage_b(self, b, imp_b):
        print_( "eaten by %s" % b.char.string)
        self.get_eaten_by(b.char)

                

    def get_unhit_by_b(self, b, stop_check):
        if not self.snapped:
            return
        if b.obj_gr == c_._OG_WALLS:
            w_id = self.stor.walls.index(b)
            wb = self.walls_below
            if wb[w_id]:
                wb[w_id] = False
                self.n_walls_below -= 1

    def get_eaten_by(self, char):
        assert(self.lock_new_phase(PHASE_EATEN))
        self.phase_lock.release()
        #we do this not to stop previous threaded phases,
        #but to make sure no future phase is running
        
##        print_( "EATEN" )
        self.eatable = False
        self.collidable = False
        if char:
            self.make_affect(char)
        self.lifetime = None
        self.gm.start_thread(self.cycle_fade, ())

    def cycle_fade(self):
        assert(self.lock_new_phase(PHASE_THR_FADING))
        self.phase_lock.release()

        delta = 0.1
        for i in range(255, 100, -50):
            if self.need_emergently_exit_phase(PHASE_THR_FADING): return
            self.alpha = i
            self.phase_lock.release()
            
            time.sleep(delta)

        if not self.gm.client_side():
            self.gm.new_event(c_._EVENT_BOUNTY_IDLE, (self.id, 0))

        self.gm.some_thread_stopped()


    def make_idle(self):
        assert(self.lock_new_phase(PHASE_IDLE))
        self.idle = True
        self.phase_lock.release()

    def make_affect(self, char):
        if getattr(bounty_effect, "make_affect_%s" % AFF_BNT[self.type])(self, char):
            self.now_sound = self.stor.sndf.get_sound(self.sl_effect[self.type])

