from ball import ball
import constants as c_
import time
from common_ import print_
from random import random, randint
from img_holder import img_holder
import bounty as M_bounty


__DEBUG__ = False

DEF_BOUNTIES_INTERVAL_ = (5.0 / 1.0, 7.0 / 1.0) #in seconds
BOUNTIES_INTERVAL_ = None
if __DEBUG__:
    DEF_BOUNTIES_INTERVAL_ = (0, 0) #in seconds
UUU = (2, 17) #in seconds
DEF_SPEED_BOUND = 0.023


def sign_b_as_a(a, b):
    if a > 1e-4:
        return abs(b)
    elif a < -1e-4:
        return -abs(b)
    else:
        return b
    
def post_exchange(stor, dxy_from_net_stor):
    clouds = stor.clouds
    res_dxy = dxy_from_net_stor
    i = 0
    for c in clouds:
        dir = [res_dxy[i], res_dxy[i+1]]
        c.speed = map(sign_b_as_a, c.speed, dir)
        i += 2
    

class cloud(ball):
    def __init__(self, obj_group, stor, x, y,
                 img_gr = c_._IG_CLOUD, img_id = 0,
                 friction = 0.1, solidity = 1.0,
                 string = "CLOUD", def_size = c_._WH_CLOUD,
                 collidable = False,
                 autorun = True,
                 gm = None,
                 id = None):
        ball.__init__(self, obj_group, stor, x, y,
                      img_gr, img_id,
                      friction, solidity,
                      string, def_size,
                      collidable)

        self.gm = gm
        self.id = id
        assert(id is not None)
        self.speed_bound = DEF_SPEED_BOUND
        self.captured = [[False] * len(self.stor.p1.char.balls),
                         [False] * len(self.stor.p2.char.balls)]
        self.belong = None

        if (autorun and (not gm.client_side())):
            if gm.server_side():
                self.my_out = gm.net_stor.my_out_vals
                self.my_out_lock = gm.net_stor.my_out_lock

            gm.start_thread(self.cycle, ())

    def bound_accel(self, dir):
        bnd = self.speed_bound
        if dir[0] > bnd:
            dir[0] = bnd
        if dir[0] < -bnd:
            dir[0] = -bnd
        if dir[1] > bnd:
            dir[1] = bnd
        if dir[1] < -bnd:
            dir[1] = -bnd


    def cycle(self):
        def rand((x, y)):
            if (random() > 0.7):
                return [(random() - 0.5) * k / 5.0, (random() - 0.5) * k / 5.0]
            else:
                return [x, y]

        def gen_next_bounty_time():
            global BOUNTIES_INTERVAL_
            r = BOUNTIES_INTERVAL_
            if __DEBUG__:
                BOUNTIES_INTERVAL_ = UUU
            return randint(int(r[0]), int(r[1]))

        #adhere to global timescale
        global BOUNTIES_INTERVAL_
        scale_f = lambda x: (x / self.gm.stor.settings.timescale)
        BOUNTIES_INTERVAL_= map(scale_f, DEF_BOUNTIES_INTERVAL_)

        #####
        delta = 0.2
        k = self.speed_bound
        dir = [0.0, 0.0]
        ddir = [0.0, 0.0]
        next_bounty_time = gen_next_bounty_time()
        curtime = time.time()
        while 1:
            t = time.time()
            next_bounty_time -= (t - curtime)
            curtime = t

            if next_bounty_time < 0:
                bounty_type = M_bounty.gen_rand_type()
                self.gm.new_event(c_._EVENT_NEW_BOUNTY, (self.id, bounty_type))
                next_bounty_time = gen_next_bounty_time()

            
            ddir = rand(ddir)
            dir = map(lambda x, y: x+y, dir, ddir)
            self.bound_accel(dir)

            if self.gm.server_side():        
                spd = dir[:] 
                index = 2 * self.id
                self.my_out_lock.acquire()
                self.my_out.cloud_dxy[index], self.my_out.cloud_dxy[index + 1] = spd
                self.my_out_lock.release()
            else:
                assert(not self.gm.client_side()) # we shouldn't run this cycle at clientside
                self.speed = map(sign_b_as_a, self.speed, dir)
                

            
            time.sleep(delta)
            if self.gm.threads_must_stop:
                break
        self.gm.some_thread_stopped()
        
        

    def check_capture(self):
        if self.belong:
            pl = self.belong.player_id - 1
            for it in self.captured[pl]:
                if it:
                    return
            self.belong = None

        for pl in range(2):
            capt = False
            for it in self.captured[pl]:
                if it:
                    capt = True
                    break
            if capt:
                self.belong = getattr(self.stor, "p%d" % (pl + 1))

 
    def get_hit_by_b(self, b, stop_check):
        if ((b.obj_gr in c_._OGs_PLAYERS) and b.attached and
            b.recognizable_by_clouds()):
            pl = b.char.player_id - 1
            if not self.captured[pl][b.id]:
                self.captured[pl][b.id] = True
                self.check_capture()

    def get_unhit_by_b(self, b, stop_check):
        if b.obj_gr in c_._OGs_PLAYERS:
            pl = b.char.player_id - 1
            if self.captured[pl][b.id]:
                self.captured[pl][b.id] = False
                self.check_capture()

    def render(self, stor, visible):
        ball.render(self, stor, visible)
        if self.belong:
            ball.render(self, stor, visible, self.get_capture_img_data)

    def get_capture_img_data(self):
        ih = img_holder(c_._IG_CLOUD, self.belong.player_id)
        return self.stor.imgf.get_image(ih)
        
