from gm_dialog import gm_dialog
from vcg_msgboard import vcg_msgboard
from vcg_dialog import vcg_dialog
from vc_button_text import vc_button_text
from vcg_scrolled_option_list import vcg_scrolled_option_list
from vcg_entitled_board import vcg_entitled_board
from vcg_scroll import vcg_scroll
from pygame.constants import K_RETURN, K_ESCAPE, K_F5
import pygame.constants
from img_holder import img_holder
from udp_client import udp_client
import constants as c_

class vcg_join_server(vcg_dialog):
    def __init__(self, daddy, stor, handler):
        vcg_dialog.__init__(self, daddy, stor, handler)
        self.btn_exit.hotkeys = (K_ESCAPE,) #excluded K_RETURN

    def add_items(self):
        vcg_dialog.add_items(self)

        add = self.items.append

        add(vcg_msgboard((("Choose a server", 60),),
                         self, self.stor, self.handler,
                         prc_area = (15, 2, 70, 20), board_img_id = 1))

        add(vc_button_text("BTN_REFRESH", K_F5, self, "Refresh", 60, 
                           prc_area = (82, 45, 17, 15),
                           button_stretched = True))

        add(vc_button_text("BTN_START", K_RETURN, self, "Join", 75, 
                           prc_area = (82, 70, 17, 15),
                           button_stretched = True))

        board = vcg_entitled_board(("", 100, 18, 60),
                                   self, self.stor, self.handler,
                                   prc_area = (1, 20, 80, 75))
        add(board)
        self.board = board

        servers = vcg_scrolled_option_list("OL_SERVERS", 5, (),
                                         board, self.stor, self.handler,
                                         prc_area = (0, 0, 100, 100),
                                         arrangement = "VERTICAL",
                                         arrng_proport = [95, 10],
                                         optlist_arrng_proport = [12, 88],
                                         init_choice_id = 0
                                         )
        self.servers = servers
        board.board_items.append(servers)

    def new_session(self):
        self.servers.dynamic_reload((), init_choice_id = None)
        vcg_dialog.new_session(self)

class gm_join_server(gm_dialog):
    def __init__(self, stor):
        gm_dialog.__init__(self, stor)

    def create_gui(self):
        return vcg_join_server(self, self.stor, self)

    def pre_session(self):
        gm_dialog.pre_session(self)
        self.udp_client = udp_client()
        self.processings_done = 0
        self.servers_data = []

    def post_processing(self):
        gm_dialog.post_processing(self)
        
        self.processings_done += 1
        if self.processings_done == 1:
            self.refresh_server_list() # we do it here to do it
                                       # *after* first frame was drawn

        #we split refreshment in 2 parts: change title, then actually refresh
        if self.need_refresh >0:
            self.need_refresh -= 1
        if self.need_refresh == 0:
            self.do_refresh_server_list()
        
    def post_session(self):
        gm_dialog.post_session(self)
        self.udp_client.close()

    def thr_find_servers(self):
        self.udp_client.find_servers()
        self.some_thread_stopped()

    def refresh_server_list(self):
        "we split refreshment in 2 parts: change title, then actually refresh"
        self.gui.servers.dynamic_reload((), init_choice_id = None)
        self.gui.board.title.board_items[0].change_text("Refreshing ...")
        self.need_refresh = 2

    def do_refresh_server_list(self):
        self.need_refresh = -1
        servers = self.udp_client.find_servers()
        self.servers_data = servers

        btns = []
        i = 0
        flag = "\\S(0.8, 0.8)\\"
        for serv in servers:
            ih = img_holder(c_._IG_SERVER_ICONS, int(serv[0]))
            if i < 9:
                hotk = getattr(pygame.constants, "K_%d" % (i + 1))
            else:
                hotk = None
            btns.append((str(i), hotk, flag + "%d. "% (i + 1) + ih.as_str() + " " + serv[1]))
            i += 1

        self.gui.servers.dynamic_reload(tuple(btns), init_choice_id = 0)

    def handle_BTN_REFRESH_click(self):
        self.refresh_server_list()

    def handle_BTN_START_click(self):
        try:
            id = int(self.gui.servers.selected.name)
        except:
            return #selected == None

        assert(len(self.servers_data) > id)
        self.chat["SERVER_HOST"] = self.servers_data[id][1]
        self.mode_next_id = c_._GM["CLIENT"]

    def handle_OL_SERVERS_value_changed(self, selected):
        if selected == None:
            s = "No servers"
        else:
            s = selected.text
        self.gui.board.title.board_items[0].change_text(s)
        

