import pygame

from game_mode import game_mode
from constants import _GM
from player1 import player1
from player2 import player2
from ci_mouse import ci_mouse
from comm_interface import comm_interface
from ball import ball
from bounty import bounty
import bounty as M_bounty
from map import map
from pygame.constants import *
import time
import constants as c_
import threading
from common_ import print_
import toolbar
import settings

class gm_playing(game_mode):

    def __init__(self, i_storage):
        game_mode.__init__(self, i_storage)
        self.scr_rect = None

        self.mouse_vis = False
        self.mouse_grab = 1
        self.events = None
        self.events_lock = threading.Lock()

    def pre_session(self):
        if self.chat["NEWGAME"] == True:
            self.start_new_game()
        self.chat.clear()

        self.stor.physics.new_session()

    def post_session(self):
        pass
#        self.chat.clear()
    
    def update_screen(self, visible):
        screen = self.stor.screen
        self.scr_rect = screen.get_rect()
        clip = pygame.Rect(self.scr_rect)
        h = toolbar.get_height(self.stor.imgf.scale)
        clip.top += h
        clip.height -= h
        self.stor.dirty.clip = clip
        self.stor.screen.set_clip(clip)

        for step in range(2):
            fname = "render_step%d" % step
            if self.stor.p1:
                getattr(self.stor.p1, fname)(self.stor, visible)
            if self.stor.p2:
                getattr(self.stor.p2, fname)(self.stor, visible)
            
        for w in self.stor.walls:
            w.render(self.stor, visible)


        for it in self.stor.bounties:
            if not it.snapped:
                it.render(self.stor, visible)

        for it in self.stor.bounties:
            if it.snapped:
                it.render(self.stor, visible)

        for c in self.stor.clouds:
            c.render(self.stor, visible)


        self.stor.dirty.clip = self.scr_rect
        self.stor.screen.set_clip(self.scr_rect)
        self.stor.toolbar.render(visible)
        
        
    def start_new_game(self):
        print "Starting new game"

        self.stor.sndf.set_music(1, force_restart = True)

        #create processes
        self.stor.process_list = []
        self.events = []

        #create players
        self.stor.p1 = player1(self, self.stor, 1)
        self.stor.p2 = player2(self, self.stor, 2)
        p1, p2 = self.stor.p1, self.stor.p2

        #init field objects
        self.stor.walls = None

        #create field objects
        self.stor.map = map()
        self.stor.map.apply(self, self.stor, p2 = True)

        #info that depends on map parameters
        self.stor.p1.set_target(self.stor.p2.char.balls[0])
        self.stor.p2.set_target(self.stor.p1.char.balls[0])

        #ci
        p1.set_comm_interface(self.get_ci_1()(p1, p2))
        p2.set_comm_interface(self.get_ci_2()(p2, p1))


    def get_ci_1(self):
        return self.stor.settings.get_ci(1)

    def get_ci_2(self):
        return self.stor.settings.get_ci(2)

    def pre_processing(self):
        for pro in self.stor.process_list:
            pro(1.0 / c_._MAX_FPS * self.stor.settings.timescale)

    def mid_processing(self):
        """called right after flip_screen();
        maybe_gameover should be called here, so user gets
        to see hp=0"""
        self.maybe_gameover()

    def post_processing(self):
        self.events_lock.acquire()

        #events - like bounties falling from clouds
        for e in self.events:
            id, param = e
            getattr(self, "handle_event_%s" % c_._S_GM_EVENTS[id])(param)
        self.events_lock.release()
        self.events = []
            

        #physics steps
        ph = self.stor.physics
        k = 1.0 / ph.def_detail * self.stor.settings.timescale
        for i in range(ph.def_detail):
            ph.make_step(k)
            
        #clean user_force
        for b in ph.balls:
            b.user_force = [0, 0]

        #other timesteps
        dt = 1.0 / c_._MAX_FPS * self.stor.settings.timescale
        for it in self.stor.bounties:
            it.make_step(dt)

    def handle_event_NEW_BOUNTY(self, (i_daddy_cloud_id, bnt_type)):
        print_( "new bounty from cloud %s" % i_daddy_cloud_id )
        M_bounty.handle_event_NEW_BOUNTY(self, (i_daddy_cloud_id, bnt_type))

    def handle_event_RELEASE_BOUNTY(self, (bounty_id, _)):
        print_( "releasing bounty %d from cloud" % bounty_id )
        self.stor.bounties[bounty_id].release_from_daddy()  
            
    def handle_event_BOUNTY_IDLE(self, (bounty_id, _)):
        self.stor.bounties[bounty_id].make_idle()


    def get_player_input(self, p):
        if p:
            data = []
            p.comm.get_events(self.stor, data)
            return data

    def handle_player_input(self, p, data):
        if not p: return
        p.apply_user_action(data) 

        req = data[-1]
        for r in range(c_._NUM_REQ):
            cmd = 1 << r
            if req & cmd:
                handler = getattr(self, "handle_request_%s" % c_._S_REQ[r])
                if handler:
                    handler(player = p)
                else:
                    print "Unknown command: %s" % cmd

    def handle_request_EXIT(self, player = None):
        self.mode_next_id = _GM["MAIN_MENU"]        
            

    def process_players_input(self):
        p1d = self.get_player_input(self.stor.p1)
        p2d = self.get_player_input(self.stor.p2)

        pygame.event.clear([KEYDOWN, KEYUP])

        self.handle_player_input(self.stor.p1, p1d)
        self.handle_player_input(self.stor.p2, p2d)


    

        
##        have_key = False
##        for e in pygame.event.get():
##            if e.type == pygame.KEYDOWN:
##                have_key = True
##                break
##
##        if have_key:
##            print "key_press: %s" % e.key
              

    def maybe_gameover(self):
        lost = [False, False]
        for p, i in [(self.stor.p1, 0), (self.stor.p2, 1)]:
            if p:
                lost[i] = int(p.info.health) <= 0

        if lost[0] or lost[1]:
            #gameover
            id0 = 0
            if self.stor.settings.get_characters(self)[1] == settings.CHR_SPIDER:
                id0 = 3
            
            print "Game over!"
            if lost[0] and lost[1]:
                self.chat["RESULT"] = ("DRAW", 2 + id0) 
            else:
                p, res = (lost[0]
                              and (self.stor.p2, 1 + id0)
                                  or (self.stor.p1, 0 + id0))
                self.chat["RESULT"] = (p.info.win_message, res)

            self.mode_next_id = _GM["GAME_OVER"]
            

        
    def new_event(self, event_name, params):
        self.events_lock.acquire()
        self.events.append((event_name, params))
        self.events_lock.release()

    def init_place_mouse(self):
        "place mouse cursor at center of world"
        cent = self.stor.physics.world_to_scr((0, 0))
        pygame.mouse.set_pos(cent)
        
    

    
    def server_side(self):
        return False

    def client_side(self):
        return False

