import common_
from common_ import copy_image
import constants as c_
import pygame
from pygame.locals import *
from pygame import Rect
import toolbar
from img_holder import img_holder
import os

class cimage:
    def __init__(self, img):
        self.img = img

    def resize_to_scale(self, scale):
        if abs(scale[0] - 1) + abs(scale[1] - 1) < 1e-3:
            return
        sz = self.img[1].size
        al = self.img[0].get_alpha()        
#        print "former size: %f, %f" % sz.size
        newsz = [int(sz[0] * scale[0]), int(sz[1] * scale[1])]
        for i in range(2):
            if newsz[i] == 0:
                if sz[i] > 0:
                    newsz[i] = 1
        im = pygame.transform.scale(self.img[0], tuple(newsz))
#        print "new size: %f, %f" % im.get_rect()
        im.set_alpha(al, RLEACCEL)
        self.img = (im, im.get_rect())
        
    def resize_to(self, size):
        if abs(self.img[1][0] - size[0]) + abs(self.img[1][1] - size[1])< 1e-3:
            return
        al = self.img[0].get_alpha()        
        im = pygame.transform.scale(self.img[0],
                                    (size[0],
                                     size[1]))
        im.set_alpha(al, RLEACCEL)
        self.img = (im, im.get_rect())

    def flip_blue_green(self):
        self.img = (common_.flip_blue_green(copy_image(self.img[0])), Rect(self.img[1]))
        
    def flip(self, x_axis, y_axis):
        self.img = (common_.my_flip(self.img[0], x_axis, y_axis), self.img[1])
        
        
        



class image_factory:
    def __init__(self, stor):
        self.groups = []
        self.rotatable = []
        self.assign_scale()
        self.stor = stor
        self.import_all_graphics()

    def assign_scale(self):
        ds, def_ds = c_._CUR_DS, c_._DEF_DS
        def f(x): return float(x)
        self.scale = (f(ds[0])/f(def_ds[0]), f(ds[1])/f(def_ds[1]))
        self.main_menu_scale = None


    def get_image(self, ih, angle = 0):
#        print self.groups[0][0]
        group_id, id = ih.img_gr, ih.img_id
        try:
            img = self.groups[group_id][id]
            if not img:
                raise IOError
        except:
            print group_id
            print id
            print "failed to get image. Group %d, image %d" % (group_id, id)
            return None

#        print "got image: %s" % img

        if abs(angle)>1e-6:
            rot = common_.my_rotate
            newimg = rot(img.img[0], angle)
            img = cimage((newimg, newimg.get_rect()))
        return img

    def get_image_alpha(self, ih, angle = 0, alpha = 0):
        img = self.get_image(ih, angle)
##        img.img[0].set_alpha(alpha)
##        return img
        img2 = common_.copy_image(img.img[0])
        img2.set_alpha(alpha, RLEACCEL)
        return cimage((img2, img2.get_rect()))


    def add_group(self, id, rotatable = False):
        print "loading image group id = %s" % id
        for i in range (id - self.size() + 1):
            self.groups.append([])
            self.rotatable.append(False)
        self.rotatable[id] = rotatable 
        
    def add_image_from_file(self, group_id, id, file, def_size = None, alpha = None, green_to_blue = False):
        gr = self.groups[group_id]
        for i in range (id - len(gr) + 1):
            gr.append(None)
            
        gr[id] = cimage(common_.load_image(file, -1))
        if def_size:
            gr[id].resize_to(def_size)
        gr[id].resize_to_scale(self.scale)
        gr[id].img[0].set_alpha(alpha, RLEACCEL)
        if green_to_blue:
            gr[id].flip_blue_green()
        return gr[id]
        
        
    def add_image_prepared(self, group_id, id, image, need_resize = True):
        gr = self.groups[group_id]
        for i in range (id - len(gr) + 1):
            gr.append(None)
        gr[id] = cimage(image)
        if need_resize:
            gr[id].resize_to_scale(self.scale)
        gr[id].img[0].convert_alpha()
        return gr[id]

    def duplicate_image(self, img_gr, img_id, img_gr2 = None):
        if img_gr2 == None: img_gr2 = img_gr
        img = self.get_image(img_holder(img_gr, img_id))
        img2 = common_.copy_image(img.img[0]), Rect(img.img[1])
        img_id2 = len(self.groups[img_gr2])
        self.add_image_prepared(img_gr2, img_id2,
                                img2, need_resize = False)
        return img_holder(img_gr2, img_id2)
        

    def rend_text(self, text, color_ignored, size):
        r"""
        I still have 'color' param since I'll probably add it
        
        text may contain scale flag.
        add "\S(XS,YS)" in the beginning, replacing XS and YS with values
        Sample:
            scale = (2, 1)
            text = "\\S%s\\" % str(scale) + "hellow, world"
            #text is rendered twice wider than usual
            
        
            
        
        text may contain extensions.
        Any loaded image may be treated as a letter.
        Sample:

        let's assume we need a text of AB&X, where & stands for groups[g][id] image:

            g, id = 1, 2
            ih = img_hodler(g, id)
            text = r"AB%sX" % ih.as_str()  #text = "AB\(1,2)\X"
        
        to include "\" in your string, use double slash:
            text = r"AB\\X"
        """
        
        text = text.upper()

        def img_holder_by_char(c):
            try:
                id = c_._S_ABC.index(c)
            except:
                id = c_._S_ABC.index(" ")
            return img_holder(c_._IG_ABC, id)
            
        #generate list of image holders for given string
        lines = text.split("\\")

        #process scale flag?
        user_scale = (1.0, 1.0)
        if len(lines) >= 2 and len(lines[1]) >= 1:
            if text[0] == "\\" and lines[1][0] == "S":
                #our line is lines[1], not lines[0]
                #because there's also an empty "" before first \\
                user_scale = eval(lines[1][1:])
                del lines[0:2] 
                text = text[text.find("\\", 2) + 1:]

        
        #first non-flag char was \\? This means first line is hyper
        hyper = False
        if text:
            if text[0] == "\\":
                del lines[0]
                hyper = True
            if text[-1] == "\\": del lines[-1]

            
        #now parts 1,3,5.. are pure chars, 2, 4, 6.. are hyperchars
           
        ih_list = []
        for line in lines:
            if hyper:
                if line == "":  #\\ -> simple slash
                    ih = img_holder_by_char("\\")
                else:
                    ih = img_holder(line) #img_hodler converts from string
                ih_list.append(ih)
            else:
                for c in line: ih_list.append(img_holder_by_char(c))

            hyper = not hyper


        #prepare surface
        scale = float(size) / 41 / 2
        scales = (scale * 0.5 * user_scale[0], scale * user_scale[1])

        n = len(ih_list)
        w = int(self.text_w * scales[0]) #text_w is set in add_abc
        h = int(self.text_w * scales[1])
        s = pygame.Surface((n * w + 1, h + 1))
        r = pygame.Rect(0, 0, w, h)

        #fill surface
        for ih in ih_list:
            t = self.get_image(ih).img[0]
            t = cimage((common_.copy_image(t), t.get_rect()))
            t.resize_to((w, h))
            t.img[1].center = r.center
            s.blit(t.img[0], t.img[1])
            r.left += w

        #finalize and return            
        ci = cimage((s, s.get_rect()))
        ci.img[0].set_colorkey(ci.img[0].get_at((0, 0)), RLEACCEL)
        return ci.img
        
        
    def import_all_graphics(self):
        print "Importing graphics..."

        #common
        self.add_abc(c_._IG_ABC)
        self.render_strings(c_._IG_STRINGS)

        #dialogs
        self.add_main_menu(c_._IG_MAIN_MENU)
        self.add_howto(c_._IG_HOWTO)
        
        self.add_group(c_._IG_MSGBOXES, False)
        for i in range(c_._IG_SZ_MSGBOXES):
            self.add_image_from_file(c_._IG_MSGBOXES, i, "msgbox%d.png" % i,
                                     c_._WH_MSGBOXES, 120)

            ###
            #single player

        self.add_group(c_._IG_DIFFIC, False)
        for i in range(c_._IG_SZ_DIFFIC):
            self.add_image_from_file(c_._IG_DIFFIC, i, "diffic%d.png" % i, c_._WH_ABC)

            ###
            #2 players

        self.add_group(c_._IG_2_PLAYERS, False)
        for i in range(c_._IG_SZ_2_PLAYERS):
            self.add_image_from_file(c_._IG_2_PLAYERS, i, "_2_players%d.png" % i,
                                     (c_._WH_ABC[0] * 20, c_._WH_ABC[1] * 4))

            ###
            #start server dialog
        self.add_group(c_._IG_SERVER_ICONS, True)
        for i in range(c_._IG_SZ_SERVER_ICONS):
            self.add_image_from_file(c_._IG_SERVER_ICONS, i,
                                     os.path.join("server_icons",
                                                  "si%d.png" % i),
                                     c_._WH_VIS_ICON)
        self.add_group(c_._IG_OPTIONLIST_SELECTION, True)
        for i in range(c_._IG_SZ_OPTIONLIST_SELECTION):
            self.add_image_from_file(c_._IG_OPTIONLIST_SELECTION, i,
                                     "sel%d.png" % i, c_._WH_ABC)

            ###
            #join game dialog
        self.add_group(c_._IG_SCROLL_ARROW, False)
        self.add_image_from_file(c_._IG_SCROLL_ARROW, 0, "arrow_up.png", c_._WH_ABC)
        self.add_image_from_file(c_._IG_SCROLL_ARROW, 1, "arrow_down.png", c_._WH_ABC)
        

            ###
            #game over
        self.add_group(c_._IG_GAME_OVER, False)
        for i in range(c_._IG_SZ_GAME_OVER):
            self.add_image_from_file(c_._IG_GAME_OVER, i, "gameover%d.png" % i,
                                     c_._WH_MSGBOXES)

            ###
            #credits
        self.add_group(c_._IG_CREDITS, False)
        for i in range(c_._IG_SZ_CREDITS):
            self.add_image_from_file(c_._IG_CREDITS, i, "credits_%d.jpg" % i)

        self.add_group(c_._IG_LOGO, False)
        for i in range(c_._IG_SZ_LOGO):
            self.add_image_from_file(c_._IG_LOGO, i, "logo%d.png" % i,
                                     (c_._WH_ABC[0] * 4 * 5 / 6, c_._WH_ABC[1] * 4))
        
##        return #now testing only dialogs
    
        #game toolbar
        self.add_backgrounds_and_toolbars(c_._IG_BACKGROUNDS, c_._IG_TOOLBARS)
        self.add_group(c_._IG_ICON_NONE, False)
        self.add_image_from_file(c_._IG_ICON_NONE, 0, "not_found.png")

        self.render_numeric_group(c_._IG_NUMBERS_BLUE, 100, (0, 0, 255), 40, 255, lower = -100)
        self.render_numeric_group(c_._IG_NUMBERS_GREEN, 100, (0, 255, 0), 40, 255, lower = -100)

        #game field
        self.render_numeric_group(c_._IG_NUMBERS_FPS, 501, (255, 255, 255), 32, 150, dot = True)

        self.add_bodies(c_._IG_BODY)

        self.add_group(c_._IG_HEAD, True)
        self.add_image_from_file(c_._IG_HEAD, 0, "head_blue.png", c_._WH_HEAD)
        self.add_image_from_file(c_._IG_HEAD, 1, "head_green.png", c_._WH_HEAD)
        self.add_image_from_file(c_._IG_HEAD, 2, "head_spdr_blue.png", c_._WH_SPDR_HEAD)
        self.add_image_from_file(c_._IG_HEAD, 3, "head_spdr_green.png", c_._WH_SPDR_HEAD)

        self.add_group(c_._IG_HEAD_FACE, True)
        for i in range(c_._IG_SZ_HEAD_FACE):
            self.add_image_from_file(c_._IG_HEAD_FACE, i, "head_0%d.png" % i, c_._WH_HEAD_FACE)

        self.add_group(c_._IG_TAIL, True)
        self.add_image_from_file(c_._IG_TAIL, 0, "tail_blue.png", c_._WH_TAIL)
        self.add_image_from_file(c_._IG_TAIL, 1, "tail_green.png", c_._WH_TAIL)

        self.add_spdr_eyes(c_._IG_SPDR_EYE)
        self.add_spidies(c_._IG_SPIDY)

        self.add_group(c_._IG_SPDR_TONGUE, True)
        self.add_image_from_file(c_._IG_SPDR_TONGUE, 0,
                                 "spdr_tongue.png", c_._WH_SPDR_TONGUE)
        
            
        
        self.add_group(c_._IG_WALL, True)
        for i in range(c_._IG_SZ_WALL):
            self.add_image_from_file(c_._IG_WALL, i, "wall%d.png" % i, c_._WH_WALL)


        self.add_group(c_._IG_CLOUD, False)
        self.add_image_from_file(c_._IG_CLOUD, 0, "cloud.png", c_._WH_CLOUD, alpha = c_._IG_AL_CLOUD)
        for i in range(c_._IG_SZ_CLOUD)[1:]:
            self.add_image_from_file(c_._IG_CLOUD, i, "cloud_capt%d.png" % (i-1), c_._WH_CLOUD, alpha = c_._IG_AL_CLOUD_CAPTURE)
            

        self.add_bounties(c_._IG_BOUNTY_321,
                          c_._IG_BOUNTY_BAD,
                          c_._IG_BOUNTY_HP,
                          c_._IG_BOUNTY_HP2)


        self.add_flowers(c_._IG_FLOWER_BLUE, switch = True)
        self.add_flowers(c_._IG_FLOWER_GREEN)

        self.add_meats(c_._IG_MEAT, 0)
        self.add_meats(c_._IG_MEAT_SPDR, 1)
        self.add_meats(c_._IG_MEAT_SPDR_EYE, 2)

        self.add_group(c_._IG_ARROW, True)
        self.add_image_from_file(c_._IG_ARROW, 0, "arrow_blue.png",
                                 c_._WH_ARROW, alpha = 150)
        self.add_image_from_file(c_._IG_ARROW, 1, "arrow_green.png",
                                 c_._WH_ARROW, alpha = 150)

        self.add_group(c_._IG_NAIL, False)
        self.add_image_from_file(c_._IG_NAIL, 0, "nail.png",
                                 c_._WH_NAIL, alpha = 220)

        self.add_group(c_._IG_RELEASABLE_BODY, True)
        self.add_image_from_file(c_._IG_RELEASABLE_BODY, 0,
                                 "bullet_blue.png", c_._WH_BODY)
        self.add_image_from_file(c_._IG_RELEASABLE_BODY, 1,
                                 "bullet_green.png", c_._WH_BODY)

        self.add_group(c_._IG_SHOT, True)
        self.add_image_from_file(c_._IG_SHOT, 0,
                                 "shot_blue.png", c_._WH_BODY)
        self.add_image_from_file(c_._IG_SHOT, 1,
                                 "shot_green.png", c_._WH_BODY)


    def add_abc(self, gr_id):
        self.add_group(gr_id, False)
        abc, rect = common_.load_image("abc.png", -1)
        w = rect.height
        s = pygame.Rect(0, 0, w, w)

        for i in range(c_._IG_SZ_ABC):
            t = pygame.Surface((w, w))
            t.blit(abc, (0, 0), s)
            s.left += w
            t.set_colorkey(t.get_at((0, 0)), RLEACCEL)
            img = cimage((t, t.get_rect()))
            img.resize_to(c_._WH_ABC)
            self.text_w = (self.add_image_prepared(gr_id, i, img.img)).img[1].width

            
        

    def render_numeric_group(self, gr_id, max, color, size, alpha, dot = False, lower = 0):
        self.add_group(gr_id, False)
        for i in range(lower, max + 1):
            s = str(i)
            if dot: s = s[:-1] + "." + s[-1]
            t = self.add_image_prepared(gr_id, i - lower,
                                        self.rend_text(s, color, size),
                                        need_resize = False)
            t.img[0].set_alpha(alpha, RLEACCEL)
            t.img[0].convert_alpha()
            

    def render_strings(self, gr_id):
        self.add_group(gr_id, False)

        i = 0
        for (s, size, alpha) in c_._RENDERED_STRS:
            self.render_given_string(gr_id, i, s, size, alpha)
            i += 1

    def render_given_string(self, gr_id, img_id, s, size, alpha):
        img = self.rend_text(s, None, size)
        if ((alpha < 255) and (alpha != None)):
            img0 = img[0]
            img0.set_alpha(alpha, RLEACCEL)
            img0.convert_alpha()
            img = (img0, img[1])
        self.add_image_prepared(gr_id, img_id, img, need_resize = False)

    def size(self):
        return len(self.groups)

    def add_bodies(self, img_gr):
        self.add_group(img_gr, True)
        self.add_image_from_file(img_gr, 0, "ball_blue.png", c_._WH_BODY)
        self.add_image_from_file(img_gr, 1, "ball_green.png", c_._WH_BODY)
        for i in range(3):
            self.add_image_from_file(img_gr, 2 + i, "spdr_leg.png",
                                     getattr(c_, "_WH_SPDR_LEG_%d" % i))
        for i in range(3):
            self.add_image_from_file(img_gr, 5 + i, "spdr_leg_symm.png",
                                     getattr(c_, "_WH_SPDR_LEG_%d" % i))


    def add_spdr_eyes(self, img_gr):
        #prepare WH
        wh0 = c_._WH_SPDR_EYE
        real_wh0 = common_.load_image("spdr_eye__.png")[1].size
        wh = []
        n = c_._IG_SZ_SPDR_EYE
        for i in range(n):
            real_wh = common_.load_image("spdr_eye_%d.png" % i)[1].size
            wh.append(tuple([int(float(wh0[i]) / real_wh0[i] * real_wh[i]) for i in range(2)]))
    
        self.add_group(img_gr, True)
        for (symm, i0) in ((False, 0), (True, n)):
            for i in range(n):
                img = self.add_image_from_file(img_gr, i + i0,
                                               "spdr_eye_%d.png" % i, wh[i])
                if symm:
                    img.flip(False, True) #flip across x axis
        
    def add_spidies(self, img_gr):
        #prepare WH
        wh0 = c_._WH_SPIDY
        real_wh0 = common_.load_image("spidy__.png")[1].size
        wh = []
        n = c_._IG_SZ_SPIDY
        for i in range(n):
            real_wh = common_.load_image("spidy_%d.png" % i)[1].size
            wh.append(tuple([int(float(wh0[i]) / real_wh0[i] * real_wh[i]) for i in range(2)]))
    
        self.add_group(img_gr, True)
        for i in range(n):
            self.add_image_from_file(img_gr, i, "spidy_%d.png" % i, wh[i])
        

    def add_backgrounds_and_toolbars(self, id_b, id_t):
        r = self.stor.screen.get_rect()
        rb = Rect(r)
        rb.height -= toolbar.get_height(self.scale) #rect backgr

        rt = Rect(r)
        rt.height = toolbar.get_height(self.scale) #rect toolbar
    
        self.add_group(id_b, False)
        for i in range(c_._IG_SZ_BACKGROUNDS):
            img = cimage(common_.load_image("background%d.jpg" % i, None))
            img.resize_to(rb.size)
            self.add_image_prepared(id_b, i, img.img, need_resize = False)

        self.add_group(id_t, False)
        trans = [None] * 4 + [-1]
        ext = ["jpg"] * 4 + ["png"]
        assert(len(trans) == c_._IG_SZ_TOOLBARS) 
        for i in range(c_._IG_SZ_TOOLBARS):
            img = cimage(common_.load_image("toolbar%d.%s" % (i, ext[i]),
                                            trans[i]))
            img.resize_to(rt.size)
            self.add_image_prepared(id_t, i, img.img, need_resize = False)

    def add_main_menu(self, gr_id):
        r = self.stor.screen.get_rect()
        self.add_group(gr_id, True)
        gr = self.groups[gr_id]

        img = cimage(common_.load_image("mm.jpg", 0))
        scl = (float(r.width) / img.img[1].width, float(r.height) / img.img[1].height) 
        self.main_menu_scale = scl
        
        def add(file, img_id):
            img = cimage(common_.load_image(file, 0))
            img.resize_to_scale(scl)
            self.add_image_prepared(gr_id, img_id, img.img, need_resize = False)

        def make_transparent(dest, cmp_with):
            b, a = dest, cmp_with
            for x in range(0, b.get_rect().width):
                for y in range(0, b.get_rect().height):
                    if b.get_at((x, y)) == a.get_at((x, y)):
                        b.set_at((x, y), 0)

        add("mm.jpg", 0)
        
        for i in range(c_._IG_SZ_MAIN_MENU)[1:]:
            add("mm__%d.png" % (i - 1), i)
##            make_transparent(gr[i].img[0], gr[0].img[0])
           

    def add_bounties(self, gr_id_123, gr_bad_id, gr_hp_id, gr_hp2_id):
        self.add_group(gr_id_123, False)
        alphas = [255] * c_._IG_SZ_BOUNTY_321
        for i in range(c_._IG_SZ_BOUNTY_321):
            self.add_image_from_file(gr_id_123, i, "bnt_cnt_%d.png" % (i+1),
                                     c_._WH_BOUNTY_321, alphas[i])
        
        self.add_group(gr_bad_id, False)
        for i in range(c_._IG_SZ_BOUNTY_BAD):
            self.add_image_from_file(gr_bad_id, i, "bounty_bad_%d.png" % i, c_._WH_BOUNTY)
        
        self.add_group(gr_hp_id, False)
        for i in range(c_._IG_SZ_BOUNTY_HP):
            self.add_image_from_file(gr_hp_id, i, "bounty_hp_%d.png" % i, c_._WH_BOUNTY)
        
        self.add_group(gr_hp2_id, False)
        for i in range(c_._IG_SZ_BOUNTY_HP2):
            self.add_image_from_file(gr_hp2_id, i, "bounty_hp2_%d.png" % i, c_._WH_BOUNTY)
        
        
 
    def add_howto(self, gr_id):
        self.add_group(gr_id, False)
        r = self.stor.screen.get_rect()
        colorkey = [None] + [-1] * (c_._IG_SZ_HOWTO - 1)
        for i in range(c_._IG_SZ_HOWTO):
            img = cimage(common_.load_image("howto_mask%d.png" % i, colorkey[i]))
            img.resize_to(r.size)
            self.add_image_prepared(gr_id, i, img.img, need_resize = False)
      
 
    def add_flowers(self, gr_id, switch = False):
        "switch means: change green and blue components, and flip vertically"
        def load(n):
            img = cimage(common_.load_image("flower%d.png" % n, -1))
            img.resize_to(c_._WH_FLOWER)
            img.resize_to_scale(self.scale)
            if switch:
                img.flip_blue_green()
                img.flip(True, False) #only vertically
            return img

        self.add_group(gr_id, False)
        for i in range(c_._IG_SZ_FLOWER):
            self.add_image_prepared(gr_id, i, load(i).img, need_resize = False).img            
        
##        self.add_group(gr_id, False)
##        prev = self.add_image_prepared(gr_id, 0, load(0).img, need_resize = False).img
##        for i in range(c_._IG_SZ_FLOWER)[1:]:
##            next = load(i).img
##            for j in range(11)[1:]:
##                tmp = copy_image(prev[0])
##                next[0].set_alpha(int((j-1) / 9.0 * 255.0))
##                tmp.blit(next[0], (0, 0))
##                self.add_image_prepared(gr_id, (i-1) * 10 + j, (tmp, tmp.get_rect()),
##                                        need_resize = False)
##            prev = next

    def add_meats(self, gr_id, meat_id):
        filename = "meat%d.png" % meat_id
        self.add_group(gr_id, True)
        h = c_._WH_MEAT_MIN[1]
        for w in range(c_._WH_MEAT_MIN[0], c_._WH_MEAT_MAX[0] + 1):
            self.add_image_from_file(gr_id, w, filename, (w / 2, h))
