from vis_comp import vis_comp
from img_list import img_list
from snd_list import snd_list
from snd_holder import snd_holder
import constants as c_

class vc_button(vis_comp):
    (_IMG_PASSIVE,
     _IMG_ACTIVE,
     _IMG_CLICKED,
     ) = range(3)

    (_SND_ACTIVE,
     _SND_CLICKED,
     ) = range(2)
     
    _IDENTIFIED_SEND = None
     
    
    def __init__(self, name, hotkeys, group, images, RM = vis_comp._RM_ON_CHANGE,
                 prc_area = None,
                 handler = None,
                 stretched = False):
        """__init__(self, name, hotkeys, group, images, RM = vis_comp._RM_ALWAYS)
        
        hotkeys is either a tuple of keys, or a single key, or None

        'images' sets passive, active, and clicked pics.
        should be either a triple of img_holders,
        or one img_holder meaning this is ih_passive and 2 others are next"""
        il = img_list(group.imgf)
        if type(images) == tuple:
            for ih in images:
                il.append(ih)
        else:
            il.load_group(images.img_gr, start = images.img_id, size = 3)

        sl = snd_list(group.sndf, (c_._SG_BTN, 0, 2))

        vis_comp.__init__(self, group, il, sl, RM, prc_area = prc_area,
                          stretched = stretched)
        self.name = name

        if handler == None: handler = group.get_child_handler()
        try:
            self.handle_on_click = getattr(handler, "handle_%s_click" % self.name)
        except:
            self.handle_on_click = self._IDENTIFIED_SEND
            self.handler = handler


        if type(hotkeys) != tuple:
            hotkeys = (hotkeys,)
        self.hotkeys = hotkeys

        self.active = False
        self.clicked = False

    def on_click(self):
        if self.handle_on_click == self._IDENTIFIED_SEND:
            self.handler.handle_IDENTIFIED_click(self)
        else:
            self.handle_on_click()
            

    def on_got_mouse(self, pos):
        self.active = True
        self.img_list.set_active(self._IMG_ACTIVE)
        self.snd_list.set_active(self._SND_ACTIVE)
        self.do_sound()
        
        
    def on_lost_mouse(self):
        if self.clicked:
            self.become_clicked_up(False)
        self.become_passive()

    def on_mouse_down(self, button, pos):
        if button == 0:
            self.become_clicked_down()
            
    def on_mouse_up(self, button, pos):
        if button == 0:
            success = self.has_mouse
            self.become_clicked_up(success)
            if not success:
                self.on_lost_mouse()
            
    def on_key_down(self, key):
        if key in self.hotkeys:
            self.become_clicked_down()
            
    def on_key_up(self, key):
        if key in self.hotkeys:
            self.active = True
            self.become_clicked_up(True)
            
    def become_passive(self):
        if self.active:
            self.active = False
            self.img_list.set_active(self._IMG_PASSIVE)

    def become_clicked_down(self):
        if not self.clicked:
            self.clicked = True
            self.img_list.set_active(self._IMG_CLICKED)
            self.snd_list.set_active(self._SND_CLICKED)
            self.do_sound()
        
    def become_clicked_up(self, activate):
        if self.clicked:
            self.clicked = False
            if activate:
                self.img_list.set_active(self._IMG_ACTIVE)
                self.on_click()
            if self.active:
                if not self.has_mouse:
                    self.become_passive()

    def new_session(self):
        vis_comp.new_session(self)        
        self.clicked = False
        self.active = False
        self.img_list.set_active(self._IMG_PASSIVE)
        self.img_list.active_changed = True
