from vc_button import vc_button
from img_list import img_list
from img_holder import img_holder
import constants as c_
from vis_comp import vis_comp

_IG = c_._IG_STRINGS_ON_THE_FLY

class vc_button_text(vc_button):
    _DONT_CHANGE = -1
    def __init__(self, name, hotkeys, group, text, size, alpha = None,
                 prc_area = None, RM = vis_comp._RM_ON_CHANGE,
                 button_stretched = False):
        """__init__(self, name, hotkeys, group, text, size, alpha, img_gr):
        
        size is either a triple (passive, active, clicked),
        or a value for all 3 (20..180)

        alpha is either a triple (passive, active, clicked),
        or a value for all 3 (0..255 or None)

        button_stretched is NOT the same is stretched,
        since we have 3 different images: passive, active, clicked,
        and we don't want them stretch to same rect:
        we need to preserve their relative sizes
        """
        self.imgf = group.imgf
        self.button_stretched = button_stretched
        
        ih_passive_text = self.pre_render_text(text,
                                               self.get_sizes(size),
                                               self.get_alphas(alpha),
                                               _IG)
        
        vc_button.__init__(self, name, hotkeys, group, ih_passive_text,
                           RM = RM, prc_area = prc_area)

    def reset_area(self, prc_area = None):
        vc_button.reset_area(self, prc_area)
        if self.button_stretched:
            self.resize_proportionally_to(self.calc_area().size)
            
    def resize_proportionally_to(self, size):
        for i in range(3):
            ih = self.img_list[i]
            k = float(self.sizes[i]) / self.sizes[0]
            sz = map(lambda x: int(x * k), size)
            self.imgf.groups[ih.img_gr][ih.img_id].resize_to(sz)

    def get_sizes(self, size):
        if type(size) != tuple:
            size = (size, int(size * 1.2), int(size * 1.2))
        return size

    def get_alphas(self, alpha):
        if type(alpha) != tuple:
            alpha = (alpha, ) * 3
        return alpha        

    def pre_render_text(self, text, sizes, alphas, img_gr):
        """pre_render_text(self, text, sizes, alphas, img_gr)

        renders given text in passive, active and clicked states,
        and appends to imgf.groups[img_gr] """

        self.text, self.sizes, self.alphas = text, sizes, alphas
        self.passive_text_img_id, self.img_gr = len(self.imgf.groups[img_gr]), img_gr

        return self.re_render_text()
        
    def re_render_text(self):
        n = self.passive_text_img_id
        for i in range(3):
            self.imgf.render_given_string(self.img_gr, n + i,
                                         self.text, self.sizes[i], self.alphas[i])
        return img_holder(self.img_gr, n)
        
    def get_shift(self):
        "in percents of self size"
        if not self.clicked:
            return (0, 0)
        else:
            ih = self.img_list[self.img_list.active_id]
            sz = self.imgf.groups[ih.img_gr][ih.img_id].img[1].size
            return (10 * sz[1] / sz[0], 5) 
            
     
    def change_text(self,
                    text = _DONT_CHANGE,
                    sizes = _DONT_CHANGE,
                    alphas = _DONT_CHANGE):
        "pass _DONT_CHANGE for a value to keep it"
        old_v = (self.text, self.sizes, self.alphas)
        
        if text   != self._DONT_CHANGE: self.text = text
        if sizes  != self._DONT_CHANGE: self.sizes = self.get_sizes(sizes)
        if alphas != self._DONT_CHANGE: self.alphas = self.get_alphas(alphas)
        
        new_v = (self.text, self.sizes, self.alphas)

        changed = False
        for i in range(3):
            if old_v[i] != new_v[i]:
                changed = True
                break

##        print "changed btn_text"
##        print "old:", old_v
##        print "new:", new_v
##

        if changed:
            self.re_render_text()
        
        
        
        
        


