from math import *
import pygame
import copy
import toolbar
from common_ import print_

class physics:

    def_detail= 5 #non-displayed physical steps in a loop
    #not every moment of game is displayed, because it would be too slow
    #so, we "slow" the physical parameters like speed
    #but run make_step procedure many times between to screen.flips
    #if not use this trick, we may loose collisions

    def_bump = 1.0
    # 1 means walls are absolutely elastic
    # 0 means walls take all your impulse, so the hit is non-elastic
    # intermediate values are possible

    def __init__(self, i_screen, i_stor):
        self.screen = i_screen
        self.stor = i_stor
        self.balls = []
    
        #prepare screen
        (fld, original) = [copy.copy(i_screen.get_rect()) for i in range(2)]
        fld.height -= toolbar.get_height(self.stor.imgf.scale)
        fld.bottom = original.bottom
        
        self.scr_r = fld
        self.rx = 1.0
        self.ry = self.rx / self.scr_r.width * self.scr_r.height
##        self.ry = 0.72
##        print self.ry
        
        
    def world_to_scr(self, coord):
        s = self.scr_r
        return ((coord[0] / self.rx / 2.0 + 0.5) * s.width + s.left,
                (coord[1] / self.ry / 2.0 + 0.5) * s.height + s.top)

    def scr_to_world(self, coord):
        s = self.scr_r
        return (((coord[0] - s.left + 0.0)/s.width - 0.5) * 2.0 * self.rx,
                ((coord[1] - s.top + 0.0)/s.height - 0.5) * 2.0 * self.ry)


    def new_session(self):
        balls = []
        balls.extend(self.stor.walls)
        balls.extend(self.stor.clouds)
        if self.stor.p1: balls.extend(self.stor.p1.char.balls)
        if self.stor.p2: balls.extend(self.stor.p2.char.balls)
        self.all_balls = balls

##    def collide(self):
##        balls = self.balls
##        for i in range(len(balls)):
##            for j in range(i+1, len(balls)):
##                d = balls[i].r + balls[j].r
##                if abs(balls[j].crd[1] - balls[i].crd[1]) > d: continue
##                if abs(balls[j].crd[0] - balls[i].crd[0]) > d: continue
##                balls[i].get_hit_by_b(balls[j], False)
        
    def collide(self):
        balls = [it for it in self.balls if it.collidable]
        crd = tuple([it.crd for it in balls])
        r = tuple([it.r for it in balls])

        balls2 = [it for it in self.balls if ((not it.collidable) and
                                              (it.__class__.__name__ in ("bounty", "cloud")))]
        crd2 = tuple([it.crd for it in balls2])
        r2 = tuple([it.r for it in balls2])
        for i in range(len(balls)):
            for j in range(i+1, len(balls)):
                d = r[i] + r[j]
                if abs(crd[j][1] - crd[i][1]) > d: continue
                if abs(crd[j][0] - crd[i][0]) > d: continue
                balls[i].get_hit_by_b(balls[j], False)

        for i in range(len(balls)):
            for j in range(len(balls2)):
                d = (r[i] + r2[j])*0.85
                if ((abs(crd2[j][1] - crd[i][1]) < d) and
                    (abs(crd2[j][0] - crd[i][0]) < d)):
                    balls2[j].get_hit_by_b(balls[i], True)
                else:
                    balls2[j].get_unhit_by_b(balls[i], True)
        
##    def collide(self):
##        balls = self.balls
##        crd = tuple([it.crd for it in balls])
##        r = tuple([it.r for it in balls])
##        i = -1
##        lb = len(balls)
##        while i < lb - 1:
##            i += 1
##            j = i
##            while j < lb - 1:
##                j += 1
##                d = r[i] + r[j]
##                if abs(crd[j][1] - crd[i][1]) > d: continue
##                if abs(crd[j][0] - crd[i][0]) > d: continue
##                balls[i].get_hit_by_b(balls[j], False)
            
    def get_meats(self):
        meats = []

        if self.stor.p1: meats.extend(self.stor.p1.char.meats)
        if self.stor.p2: meats.extend(self.stor.p2.char.meats)

        return meats

    def frict(self, balls, scale):

        #apply force of friction
        for it in balls:
            mul = pow((1 - it.friction) * 0.997, scale) 
            it.speed[0] = it.speed[0] * mul
            it.speed[1] = it.speed[1] * mul
##            it.speed[0] = it.speed[0] * (1 - it.friction * scale) 
##            it.speed[1] = it.speed[1] * (1 - it.friction * scale) 
    
    def make_step(self, scale = 1):
##        print_ ("physics step. speeds of heads: ")
##        print_ ("%0.5f %0.5f" % tuple(self.stor.p1.char.balls[0].speed))
##        print_ ("%0.5f %0.5f" % tuple(self.stor.p2.char.balls[0].speed))
        
        self.balls = self.all_balls + self.stor.bounties #because bounties may disappear
        balls = self.balls
        meats = self.get_meats()

        #apply user force

        for it in balls:
            it.speed[0] = it.speed[0] + (it.accel[0] + it.user_force[0]) * scale
            it.speed[1] = it.speed[1] + (it.accel[1] + it.user_force[1]) * scale

        #apply stretches
        for it in meats:
            it.take_affect(scale)

        #stick sticked
        for it in balls:
            if it.sticked:
                it.speed = [0, 0]

    


        #collide balls with each other
        #for optimizing this we perform a simple check here
        self.collide()                    


        self.frict(balls, scale)


        #collide with walls
        bump = physics.def_bump
        bumped = 0
        for it in balls:
            if (it.crd[0]) < -self.rx + it.r:
                if it.speed[0] < 0:
                    it.speed[0] = abs(it.speed[0] * bump)
                    bumped += it.speed[0]
            if (it.crd[0]) > self.rx - it.r:
                if it.speed[0] > 0:
                    it.speed[0] = -abs(it.speed[0] * bump)
                    bumped -= it.speed[0]
            if (it.crd[1]) < -self.ry + it.r:
                if it.speed[1] < 0:
                    it.speed[1] = abs(it.speed[1] * bump)
                    bumped += it.speed[1]
            if (it.crd[1]) > self.ry - it.r:
                if it.speed[1] > 0:
                    it.speed[1] = -abs(it.speed[1] * bump)
                    bumped -= it.speed[1]
            if bumped > 1e-5:
                it.hit_border(bumped)

           
        ###move objects
        for it in balls:
            it.crd = (it.crd[0] + it.speed[0] * scale, it.crd[1] + it.speed[1] * scale)

        #snap snapped bounties
        for it in self.stor.bounties:
            if it.snapped:
                it.snap()

                
##        print_( "   %0.5f, %0.5f" % tuple(self.stor.clouds[0].speed) )
##        if len(self.stor.bounties) > 0:
##            print_( "b: %0.5f, %0.5f" % tuple(self.stor.bounties[0].speed) )
           
if __name__ == "__main__":
    import pygame
    b = pygame.display.set_mode((640, 480))
    a = physics(b)
    print a.ry
    print a.world_to_scr((0, 0))
    
