import pygame
from common_ import *
import constants as c_
from my_maths import *
from math import *
from img_holder import img_holder
from snd_holder import snd_holder

class ball(pygame.sprite.Sprite):
    _SH_HIT = snd_holder(c_._SG_HITS, 1)
    _SH_HIT_BRD = snd_holder(c_._SG_HITS, 1)
   

    def __init__(self, obj_group, stor, x, y, img_gr = c_._IG_BODY, img_id = 0,
                 friction = 0.1, solidity = 1.0, string = "anonymous ball",
                 def_size = (0, 0), collidable = True,
                 vulnerability = 1.0,
                 visible = True):
        pygame.sprite.Sprite.__init__(self)
        self.stor = stor
        self.obj_gr = obj_group
        self.string = string
        self.def_size = def_size

        #graphics
        if not img_gr:
            img_gr = c_._IG_BODY
        self.ih_main = img_holder(img_gr, img_id)
        self.imgf_get_image = self.stor.imgf.get_image
        self.imgf_get_image_alpha = self.stor.imgf.get_image_alpha
        self.cached_rect = None

        #sounds
        self.sh_hit = self._SH_HIT
        self.sh_hit_border = self._SH_HIT_BRD
        self.now_sound = None
        

        #physical params
        #
        rect = self.get_rect(rotated = False)
        
        self.r = float(def_size[0]) / c_._DEF_DS[0] * c_._CUR_DS[0] / 2.0
        self.r = stor.physics.scr_to_world((self.r + stor.screen.get_rect().width / 2.0, 0))[0]
	
            
        
##        print_( "ball \"%s\"'s r = %s" % (self.string, self.r) )
##        print_( collidable )
        self.crd = (x, y)
        self.speed = [0, 0]
        self.accel = [0, 0]
        self.force = [0, 0]

        self.user_force = [0, 0]
        self.friction_force = [0, 0]

        self.collidable = collidable
        self.friction = friction
        self.solidity = solidity
        self.mass = self.r * self.r * self.solidity
        self.sticked = False
        self.sticky = False
        self.vulnerability = vulnerability
        self.visible = visible

    def render(self, stor, visible, datafunc = None, drawing_invisible = False):
        if not self.visible:
            if not drawing_invisible:
                return
        if not visible:
            if self.cached_rect:
                blit_(stor, None, self.cached_rect, visible)
                self.cached_rect = None
            return

        if not datafunc: datafunc = self.get_image_data
        img_d = datafunc()
        rect = pygame.Rect(img_d.img[1])
        rect.center = stor.physics.world_to_scr(self.crd)
        blit = drawing_invisible and blit_unerase or blit_
        blit(stor, img_d.img[0], rect, visible)
        self.cached_rect = rect
##        if visible:
##            stor.screen.blit(img_d.img[0], rect)
##        else:
##            stor.screen.blit(stor.background, rect, rect)

    def get_rect(self, rotated = True):
        return self.get_image_data(rotated).img[1]

    def get_image_data(self, rotated = True):
        ang = rotated and self.get_image_angle() or 0
        return self.imgf_get_image(self.ih_main, angle = ang)

    def get_image_angle(self):
        return 0

    def get_unhit_by_b(self, b, stop_check):
        pass

    def get_hit_by_b(self, b, stop_check):
        if not (self.collidable and b.collidable):
            return
##        if not (self.collidable and b.collidable):
##            print_( "colliding non-collidable" )
##            print_( " %s" % self.string )
##            print_( " %s" % b.string )
##            if stop_check:
##                print_( "from second cycle" )
        a = self

        ac, bc = a.crd, b.crd
        d = a.r + b.r
        if abs(ac[1] - bc[1]) > d: return
        if abs(ac[0] - bc[0]) > d: return 
        
        d = d*d
        if (ac[1] - bc[1])*(ac[1] - bc[1]) + (ac[0] - bc[0])*(ac[0] - bc[0]) > d:
            return
        
        
        va = vect(a.crd)
        vb = vect(b.crd)
        if va.dist_to_b(vb) > a.r + b.r:
            return

        #collision found
        vspb = vect(b.speed)
        vba = vect([a.crd[0] - b.crd[0],
                    a.crd[1] - b.crd[1]])

        vspa= vect(a.speed)
        vab = vect([b.crd[0] - a.crd[0],
                    b.crd[1] - a.crd[1]])

        #spc - collinear part, spp - perpendicular part
#        print "vsp = %s,      vba = %s" % (vsp, vba)
        spcb, sppa = vspb.decomposed_by_b(vba)
        spca, sppb = vspa.decomposed_by_b(vab)

        #they may have different weights
        k = b.mass / a.mass
#        print "k = %s" % k
        spcb_, spca_ = (spcb, spca)
        
        add_spca = spca_.mult_by_x((1.0 - k) / (1.0 + k)).add_b(spcb_.mult_by_x(2.0 * k / (k + 1.0)))
        add_spcb = spca_.mult_by_x(2.0 / (1.0 + k)).add_b(spcb_.mult_by_x((k - 1.0) / (k + 1.0)))
#        spca = spca.mult_by_x(b.mass / a.mass)
#        spcb = spcb.mult_by_x(a.mass / b.mass)



#            if stop_check:
#                self.speed = [0.0, 0.0]
#            else:

#        print spca.len()
#        print spcb.len()
        a_hit = ((spcb.len() > 1e-8)) and (spcb.cosine(vba) > 0.1)
        b_hit = ((spca.len() > 1e-8)) and (spca.cosine(vab) > 0.1)
#        print "a_hit = %s" % a_hit
#        print "b_hit = %s" % b_hit
        if a_hit or b_hit:
            ok = False
            if a_hit and b_hit:
                ok = True
            else:
                if a_hit:
                    ok = spcb.len() > spca.len() + 1e-5
                else:
                    if b_hit:
                        ok = spca.len() > spcb.len() + 1e-5
                    else:
                        print_("Impossible situation")
                        raise(IOError)
            if ok: #collision found!
                da = vect([- spca_[0] + add_spca[0], - spca_[1] + add_spca[1]])  
                db = vect([- spcb_[0] + add_spcb[0], - spcb_[1] + add_spcb[1]])  
                if abs(da[0]) > 1e-8:
                    a.speed[0] += da[0]
                if abs(da[1]) > 1e-8:
                    a.speed[1] += da[1]
                if abs(db[0]) > 1e-8:
                    b.speed[0] += db[0]
                if abs(db[1]) > 1e-8:
                    b.speed[1] += db[1]

##                print "hit!"
##                print " %s %s" % (a.string, 0)
##                print " %s %s" % (b.string, 0)
#                print " %s %s" % (a.string, a.attached)
#                print " %s %s" % (b.string, b.attached)
                self.maybe_affect_each_other(b, da.len(), db.len())
               

    def debug_angle(self, stor):
        ph = stor.physics
        c = self.get_rect().center
        s = ph.world_to_scr((0, 0))
        s1= ph.world_to_scr(self.speed)
        s = (s1[0] - s[0], s1[1] - s[1])
        s = (s[0] * 10, s[1] * 10)
        pygame.draw.line(stor.screen, (0, 0, 255), c, (c[0] + s[0], c[1] + s[1]), 1)

    def maybe_affect_each_other(self, b, imp_a, imp_b):
        #imp_a is impulse of a that it gets from b
        a = self

        for (x, y) in ((a, b), (b, a)):
##            try:
##                print "x:" 
##                print "x.sticky: %s" % x.sticky
##                print "groups: %s %s" % (x.obj_gr, y.obj_gr)
##                print "y.attached: %s" % y.attached
##
##                print [z.sticky for z in self.stor.p2.snake.balls]
##                print self.stor.p2.snake.sticky_id
##
##            except:
##                pass
            
            if x.glues_b(y):
                x.glue_b(y)
                break

        for (x, y, imp_x, imp_y) in ((a, b, imp_a, imp_b), (b, a, imp_b, imp_a)):
            if x.damages_b(y):
                x.damage_b(y, imp_y)

        a.do_my_sound(b, imp_b)
        b.do_my_sound(a, imp_b)
            

    def glues_b(self, b):
        return False
    def glue_b(self, b):
        pass

    def damages_b(self, b):
        return False
    def damage_b(self, b, imp_b):
        pass

    def recognizable_by_clouds(self):
        return True

    def do_my_sound(self, b, imp):
        if self.need_sound_when_hit_b(b):
            if imp * 100 > 1:
                self.now_sound = self.stor.sndf.get_sound(self.sh_hit)

    def need_sound_when_hit_b(self, b):
        return True

    def hit_border(self, imp):
        if imp * 100 > 1:
            self.now_sound = self.stor.sndf.get_sound(self.sh_hit_border, imp)
        


    
