import pygame

from char import char
from snake_body import snake_body
from snake_head import snake_head
from snake_tail import snake_tail
from meat import meat
from tongue import tongue
from animation import animation
from my_maths import vect
import constants as c_
from common_ import print_
from arrow import arrow
from snd_holder import snd_holder


class snake(char):
    charname = "SNAKE"
    def __init__(self, stor, player, spawn_vect, spawn_data = (0, 2)):
        """
        for Snake character,
        spawn_data is a triple (total, detached, min_body_len)

        total is the number of balls, including head and tail
        detached is initial number of balls detached
        """
        
        char.__init__(self, stor, player, spawn_vect, spawn_data)
        

    def create_body(self, spawn_vect, spawn_data):
        """
        spawn vect is a (x, y, dx, dy) tuple, passed in world coords
        spawn_data is any object, specific to a character
        """

        total, detached, self.min_body_len = spawn_data
        
        #create balls
        if 1:
            
            self.ordered_balls = [] # this list is taken care of in mark_bullets()
            bs = self.balls
            img_id = self.player_id - 1

            #calc initial coords
            crds = []
            sv = spawn_vect
            x0, y0 = sv[0], sv[1]
            dx, dy = float(sv[2]) / total, float(sv[3]) / total
            
            for i in range(total):
                crds.append((x0 + dx * i, y0 + dy * i))
                

            #head
            bs.append(snake_head(self, 0, None, None, self.obj_group, self.stor,
                                 crds[0][0], crds[0][1], c_._IG_HEAD, img_id,
                                 string = "HEAD OF PLAYER %s" % self.player_id))
            self.head = bs[0]
            prev = bs[0]

            #bodies
            count = 1
            for it in crds[1:-1]:
                bs.append(snake_body(self, count, prev, None, self.obj_group, self.stor,
                                     it[0], it[1], c_._IG_BODY, img_id,
                                     string = "BALL %d OF PLAYER %s" % (count, self.player_id)))
                curr = self.balls[-1]
                prev.next_ball = curr
                prev = curr
                count += 1

            #tail 
            bs.append(snake_tail(self, count, prev, None, self.obj_group, self.stor,
                                 crds[-1][0], crds[-1][1], c_._IG_TAIL, img_id,
                                 string = "TAIL OF PLAYER %s" % self.player_id))
            prev.next_ball = bs[-1]

            #total_attached
            self.total_attached = len(bs)
            
        #create links
        b = self.balls
        n = len(b) - detached
        for i in range(n - 1):
            self.meats.append(meat(b[i], b[i + 1]))

        
        #detach part of body
        for i in range(len(b) - detached, len(b)):
            b[i].make_detached()
        self.last_attached = b[len(b) - detached - 1]

        self.mark_bullets(True)

        #create tail arrow
        self.arrow = arrow(self.stor, self, img_id)

    def set_initial_params(self):
        pass
        

    def mark_bullets(self, force_images_change):
        "responsible of changing images of body elements"
        ob = [] #ordered balls
        
        b = self.balls[0]
        last_ball = None
        for i in range(self.min_body_len):
            b.make_non_releasable(force_images_change)
            ob.append(b)
            last_ball = b
            b = b.next_ball
        if last_ball: last_ball.make_non_releasable(True)
        #always mark the end of non-releasable group

        last_ball = None
        while b:
            b.make_releasable(force_images_change)
            ob.append(b)
            last_ball = b
            b = b.next_ball
        if last_ball: last_ball.make_releasable(True)
        #always mark the end of releasable group
            
        self.ordered_balls = ob
        
    def try_release_bullet(self):
        if self.total_attached == self.min_body_len:
            self.now_sound = self.stor.sndf.get_sound(self.sh_noammo)
            return
        b = self.last_attached
        a = b.prev_ball

        self.kill_meat(a, b)
        a.next_ball = None
        b.make_detached()
        self.last_attached = a
        print_( "detached from player %d" % self.player_id)
        self.mark_bullets(True)
        self.now_sound = self.stor.sndf.get_sound(self.sh_fire)
##        print "total attached = %d" % self.total_attached

    def get_cur_head_dir(self):
        sx, sy, sm = 0.0, 0.0, 0.0
        n = 0
        for it in self.balls:
            if it.attached:
                sx += it.crd[0] * it.mass #* it.friction
                sy += it.crd[1] * it.mass #* it.friction
                sm += it.mass #* it.friction
                n += 1
        assert(n == self.total_attached)
        sm /= float(n)
        sx /= float(n) * sm; sy/= float(n) * sm
        h = self.balls[0].crd
##        print self.player.name
##        print "sx, sy = %f, %f" % (sx, sy)
##        print "h = %f, %f" % h
        return vect([h[0] - sx, h[1] - sy]).normalized()

    def render_step0(self, stor, visible):
        self.arrow.render(stor, visible)
        for it in self.meats:
            it.render(stor, visible)
        for it in self.balls[1:]:
            it.render(stor, visible)
    def render_step1(self, stor, visible):
        self.balls[0].render(stor, visible)
##        self.render_debug_direction(stor, visible)

    def render_debug_direction(self, stor, visible):
        crd = self.balls[0].crd
        dir = self.get_cur_head_dir()
        crd2 = (crd[0] + dir[0] / 10.0, crd[1] + dir[1] / 10.0)
        a = stor.physics.world_to_scr(crd)
        b = stor.physics.world_to_scr(crd2)
        color = visible and (255, 255, 255) or (0, 0, 0)
        pygame.draw.line(stor.screen, color, a, b)
        
        
         
#        self.tongue.render(stor, visible)

    def add_initial_anim(self):
        cf = c_._IG_HEAD_FACE
        size = c_._IG_SZ_HEAD_FACE
        animation(self.stor, self.balls[0].ih_face, cf, 0,
                  size - 1, "12321" , delay = 0.1)

    def get_cursable_balls(self):
        return self.ordered_balls[self.min_body_len:]
