import constants as c_
from common_ import print_
import pygame
import os

MUS_FILES = ("dialogs.ogg", "game.ogg", "gameover.ogg")

class sound_factory:
    def __init__(self, stor):
        self.stor = stor
        self.settings = stor.settings
        self.groups = []
        self.mixer = pygame.mixer
        self.mixer.init(11025) #raises exception on fail

        self.glob_sfx_vol = float(c_._DEF_SFX_VOL) / 10.0 
        self.glob_mus_vol = float(c_._DEF_MUS_VOL) / 10.0
        
        self.import_all_sounds()

        self.muser = self.mixer.music
        self.music_id = None
        self.music =  None
        self.music_files = [os.path.join(os.getcwd(),
                                         "music",
                                         s)
                            for s in MUS_FILES]
        

    def get_sound(self, sh, stop_all = True, volume = None):
        " volume won't work, because Channel.set_volume() doesn't work :( "
        
        snd, init_v = self.groups[sh.snd_gr][sh.snd_id]
        if not snd:
            return None

        snd.set_volume(init_v * self.glob_sfx_vol)
        
        if stop_all:
            snd.stop()
        channel = snd.play()
        if volume != None:
            channel.set_volume(volume)

        if not self.settings.sound:
            channel.stop()

##        channel.stop() #disable sound anyway, because it is not ready in this version
        return channel
   

    def size(self):
        return len(self.groups)

    def add_group(self, id, rotatable = False):
        print "loading sound group id = %s" % id
        for i in range (id - self.size() + 1):
            self.groups.append([])

    def add_sound_from_file(self, group_id, id, filename,
                            simulate_missing = False,
                            init_vol = 1.0):
        ext = filename.split(".")[-1]
        fullname = os.path.join('sounds', os.path.join(ext, filename))

        gr = self.groups[group_id]
        for i in range (id - len(gr) + 1):
            gr.append(None)

        try:
            gr[id] = self.mixer.Sound(fullname)
            gr[id].set_volume(init_vol)
            if simulate_missing:
                raise
        except:
            print "    Couldn't load sound: %s" % fullname
            gr[id] = None

        gr[id] = (gr[id], init_vol)
        return gr[id]
        

    def import_all_sounds(self):
        print "Importing sounds..."

        #game dialogs
        self.add_group(c_._SG_BTN)
        for i in range(c_._SG_SZ_BTN):
            self.add_sound_from_file(c_._SG_BTN, i, "btn%d.ogg" % i)
        
        #gameplay
        self.add_group(c_._SG_HITS)
        for i in range(c_._SG_SZ_HITS):
            self.add_sound_from_file(c_._SG_HITS, i, "hit%d.ogg" % i)
        
        self.add_group(c_._SG_FIRE)
        for i in range(c_._SG_SZ_FIRE):
            self.add_sound_from_file(c_._SG_FIRE, i, "fire%d.ogg" % i)
        

        self.add_group(c_._SG_BODY)
        self.add_sound_from_file(c_._SG_BODY, 0, "attached.ogg")

        self.add_group(c_._SG_BOUNTY)
        for i in range(c_._SG_SZ_BOUNTY):
            self.add_sound_from_file(c_._SG_BOUNTY, i, "bounty%d.ogg" % i)
        
        self.add_group(c_._SG_TIME)
        self.add_sound_from_file(c_._SG_TIME, 0, "tick.ogg", init_vol = 0.2)
        
        self.add_group(c_._SG_MISC)
        for i in range(c_._SG_SZ_MISC):
            self.add_sound_from_file(c_._SG_MISC, i, "misc%d.ogg" % i)


    def set_music(self, id, force_restart = False, play_once = False):
        if id != self.music_id:
            force_restart = True

        self.music_id = id
        if force_restart:

            if self.music != None:
                self.music.stop()

            ##looping doesn't work nicely in pygame.mixer
##            self.muser.load(self.music_files[id])
##            self.muser.play(-1)
            filename = self.music_files[id]
            if not os.path.exists(filename):
                print_( "music missing: %s" % filename)
                return
                
            self.music = self.mixer.Sound(filename)
            if play_once:
                loop = 0
            else:
                loop = -1
            self.music.set_volume(self.glob_mus_vol)
            self.music.play(loop)
                        

    def set_glob_sfx_vol(self, val):
        self.glob_sfx_vol = val

    def set_glob_mus_vol(self, val):
        self.glob_mus_vol = val
        if self.music != None:
            self.music.set_volume(val)
    
