from char import char
from spider_head import spider_head
from spider_body import spider_body
from spider_tongue import spider_tongue
from spidy import spidy
import math
from math import sin, cos
from my_maths import angle_a_to_b_rad, period_to_range, czero, dist_a_to_b
from meat import meat
from img_holder import img_holder
import constants as c_
from animation import anim_rnd_pause
from snd_holder import snd_holder

_BODY_ANGLES = [x / 180.0 * math.pi for x in (25, 60, 95, 160)] #leg0, leg1, leg2, eye

class spider(char):
    charname = "SPIDER"
    uncurse_phases_delays = [1, 0, 0.6, 0, 0.1, 0]
    n_uncurse_phases = len(uncurse_phases_delays)

    _SH_FIRE = snd_holder(c_._SG_FIRE, 3)
    def __init__(self, stor, player, spawn_vect, spawn_data = None):
        """
        for Spider character,
        spawn_data is [!/!] (not know yet)
        """
        self.eye_anim = []
        
        char.__init__(self, stor, player, spawn_vect, spawn_data)
        self.stor.process_list.append(self.time_step)
        self.time_uncurse = None
        self.uncurse_phase = None
        

    def create_body(self, spawn_vect, spawn_data):
        """
        spawn vect is a (x, y, dx, dy) tuple, passed in world coords
        spawn_data is any object, specific to a character
        """
        self.bones = [] #will contain skeleton parts, to tie legs and eyes to them
        self.pieces = [] #eyes and legs
        self.eyes = []
        self.legs = []
        self.spidies = []
        self.spidy_legs = []
        self.leg_meats = []
        

        #create balls
        id = [0] #for autoincremetion; see spider_body
        
        if 1:

            # N O T E:
            #
            # order of creation matters, because dist is not always changed,
            # and used in next block
            

            self.total_attached = 0 #will subtract from this if detached on creation
                        
            img_id = self.player_id - 1

            def add(item):
                self.balls.append(item)
        
            #prepare for calculating coords
            crds = []
            x0, y0, dx, dy = spawn_vect
            angs = _BODY_ANGLES
            ang0 = angle_a_to_b_rad((0, 0), (dx, dy)) + math.pi
            n = len(angs)

            #head
            self.head = spider_head(self, id, self.obj_group, self.stor, x0, y0,
                                    img_id = img_id)
            add(self.head)

            #legs and eyes
            r = self.head.r
            big_leg_r = spider_body(self, -1, self.obj_group, self.stor, 0, 0).r
            dist = r + big_leg_r * 2.0

            def add_side(lst, sign, visible = True, collidable = True,
                         class_ = spider_body): #symmetric body
                range_ = range(n)
                if class_ == spidy:
                    range_ = [1]
                    
                img_id0 = 0
                
                if sign == 1:
                    range_.reverse()
                    img_id0 = 3
                    
                v_img_id = [2, 1, 0, 0]
                v_wh = [c_._WH_SPDR_LEG_2,
                        c_._WH_SPDR_LEG_1,
                        c_._WH_SPDR_LEG_0,
                        c_._WH_SPDR_LEG_0]
                for i in range_:
                    part = class_(self, id, self.obj_group, self.stor,
                                  x0 + dist * cos(ang0 + sign * angs[i]),
                                  y0 + dist * sin(ang0 + sign * angs[i]),
                                  img_id = v_img_id[i] + img_id0,
                                  def_size = v_wh[i],
                                  visible = visible,
                                  collidable = collidable)
                    lst.append(part)
                    self.balls.append(part)

            add_side(self.pieces, 1)
            add_side(self.pieces, -1)

            self.eyes.append(self.balls[1])
            self.eyes.append(self.balls[-1])

            self.legs.extend(self.balls[2:-1])
            self.spidy_legs = [self.legs[0], self.legs[-1]]

            #spidies
            spidy_r = spidy(self, -1, self.obj_group, self.stor, 0, 0).r
            dist = dist + spidy_r * 1.5
            add_side(self.spidies, 1,  class_ = spidy)
            add_side(self.spidies, -1, class_ = spidy)

            #tongue
            tongue_r = spider_tongue(self, -1, self.obj_group, self.stor, 0, 0).r
            dist = r + tongue_r
            self.tongue = spider_tongue(self, -1, self.obj_group, self.stor,
                                        x0 + dist * cos(ang0 + math.pi),
                                        y0 + dist * sin(ang0 + math.pi))
            self.balls.append(self.tongue)

            #skeleton
            skel_r = spider_body(self, -1, self.obj_group, self.stor, 0, 0).r / 2.0
            dist = r - skel_r
            self.dist_to_bones = dist

            add_side(self.bones, 1,  visible = False, collidable = False)
            add_side(self.bones, -1, visible = False, collidable = False)

            #mouth
            # not tied to bones,
            # we simply set it crd to self.get_mouth_crd() each frame
            mouth_crd = self.get_mouth_crd()
            self.mouth = spider_body(self, -1, self.obj_group, self.stor,
                                     mouth_crd[0], mouth_crd[1],
                                     visible = False,
                                     collidable = False)
            self.balls.append(self.mouth)
            
            

            
            #total_attached
            self.total_attached += len(self.balls)

        #create links
        if 1:
            def add_meat(ball1, ball2, img_gr, visible, elast = 10, dist_k = "AUTO"):
                m = meat(ball1, ball2, dist_k, elast = elast,
                         img_gr = img_gr,
                         visible = visible)
                self.meats.append(m)
                return m

            n = len(self.bones)

            #tie legs to bones
            for i in range(n):
                part, bone = self.pieces[i], self.bones[i]
                if part in self.eyes:
                    meat_img_gr = c_._IG_MEAT_SPDR_EYE
                else:
                    meat_img_gr = c_._IG_MEAT_SPDR

                if i < n/2: #meats must be bend symmetrically
                    t = part
                    part = bone
                    bone = t
                m = add_meat(part, bone, meat_img_gr, True, elast = 1)
                
                if self.pieces[i] in self.legs: #we can't say "part in self.legs,
                                                #because part and bone are swapped
                                                #for one of the 2 sides
                    self.leg_meats.append(m)

            #tie circle of bones
            for i in range(n):
                add_meat(self.bones[i - 1], self.bones[i], c_._IG_MEAT_SPDR_EYE, False)

            #tie spidies to legs
            for i in [0, 1]:
                m = add_meat(self.spidy_legs[i], self.spidies[i], c_._IG_MEAT_SPDR, True,
                             dist_k = 1.3, elast = 0.5)
                self.spidies[i].my_meat = m

            #tie tongue to mouth and to eye-holders
            add_meat(self.mouth, self.tongue, c_._IG_MEAT_SPDR_EYE, True, elast = 0.25)
            tm = [] #tongue meats
            tm.append(add_meat(self.bones[0], self.tongue, c_._IG_MEAT_SPDR_EYE,
                               False, elast = 0.5))
            tm.append(add_meat(self.bones[7], self.tongue, c_._IG_MEAT_SPDR_EYE,
                               False, elast = 0.5))
            self.tongue.set_tongue_meats(tm)
        
            #tie head to bones
            add_meat(self.head, self.bones[2], c_._IG_MEAT_SPDR_EYE, False)
            add_meat(self.head, self.bones[1], c_._IG_MEAT_SPDR_EYE, False)
            add_meat(self.head, self.bones[-2], c_._IG_MEAT_SPDR_EYE, False)
            add_meat(self.head, self.bones[-3], c_._IG_MEAT_SPDR_EYE, False)
            add_meat(self.head, self.bones[0], c_._IG_MEAT_SPDR_EYE, False)
            add_meat(self.head, self.bones[3], c_._IG_MEAT_SPDR_EYE, False)
            add_meat(self.head, self.bones[4], c_._IG_MEAT_SPDR_EYE, False)
            add_meat(self.head, self.bones[-1], c_._IG_MEAT_SPDR_EYE, False)

            #tie eyes to nearby non-eye bones, so mouth tends to be free
            add_meat(self.eyes[0], self.bones[1], c_._IG_MEAT_SPDR_EYE, False,
                     elast = 3, dist_k = 1.7)
            add_meat(self.eyes[1], self.bones[6], c_._IG_MEAT_SPDR_EYE, False,
                     elast = 3, dist_k = 1.7)

            
    def set_initial_params(self):
        self.stor.process_list.append(self.turn_head_to_opponent)

    def turn_head_to_opponent(self, dt):
        #[/!/]make smarter!
        cur_ang = angle_a_to_b_rad(self.head.crd, self.get_mouth_crd())
        dest_ang = angle_a_to_b_rad(self.head.crd, self.target_ball.crd)
        da = dest_ang - cur_ang
        da = period_to_range(da, -math.pi, math.pi) 
        da *= dt / 2.0
        #[/!/]bound!!

        (x0, y0) = self.head.crd
        r = self.dist_to_bones
        for bone in self.bones:
            x, y = bone.crd
            ang = angle_a_to_b_rad((x0, y0), (x, y)) + da
            newx, newy = x0 + r * cos(ang), y0 - r * sin(ang)
##            print "\nold: (%0.3f, %0.3f)" % (x, y)
##            print "new: (%0.3f, %0.3f)" % (newx, newy)
            bone.user_force[0] = newx - x
            bone.user_force[1] = newy - y
        
    def render_step0(self, stor, visible):
        for it in self.meats:
            it.render(stor, visible)
        for b in self.balls:
            if b not in self.eyes:
                b.render(stor, visible)

    def render_step1(self, stor, visible):
        for it in (self.meats[0], self.meats[7]): #eye meats
            it.render(stor, visible)
        for b in self.eyes:
            b.render(stor, visible)

    def add_initial_anim(self):
        for eye, img_id in ((self.eyes[0], 0), (self.eyes[1], 4)):
            eye.get_image_angle = eye.get_angle_to_target_ball
            a = anim_rnd_pause(self.stor, eye.ih_main, c_._IG_SPDR_EYE,
                               img_id, img_id + 3, delay = 0.1,
                               min_pause = 2, max_pause = 4)
            self.eye_anim.append(a)

    def get_mouth_crd(self):
        a = self.bones[0].crd
        b = self.bones[-1].crd
        return (a[0] + b[0]) / 2.0, (a[1] + b[1]) / 2.0

    def time_step(self, dt):
        self.mouth.crd = self.get_mouth_crd()
        self.tongue.time_step(dt)
        for sp in self.spidies:
            sp.time_step(dt)
            
    def find_attached_spidy_closest_to_target(self):
        closest, closest_dist = None, None
        for it in self.spidies:
            if it.attached:
                dist = dist_a_to_b(it.crd, self.target_ball.crd)
                if ((closest == None) or (dist < closest_dist)):
                    closest = it
                    closest_dist = dist
        return closest, closest_dist
        
    def try_release_bullet(self):
        sp, dist = self.find_attached_spidy_closest_to_target()
        if sp:
            sp.shoot(self.target_ball.crd, dist)
            self.now_sound = self.stor.sndf.get_sound(self.sh_fire)
            
    def get_cursable_balls(self):
        return [self.head]

    def got_cursed(self):
        if self.uncurse_phase == None:
            self.time_uncurse = self.uncurse_phases_delays[0]
            self.uncurse_phase = 0
            self.stor.process_list.append(self.timer_uncurse)

    def timer_uncurse(self, dt):
        self.time_uncurse -= dt
        
        if self.time_uncurse < 0:
            self.uncurse_phase += 1
            if self.uncurse_phase == self.n_uncurse_phases:
                self.uncurse_phase = None
                self.stor.process_list.remove(self.timer_uncurse)
            else:
                self.time_uncurse = self.uncurse_phases_delays[self.uncurse_phase] - self.time_uncurse
            
        
        i = self.uncurse_phase
        if i == 0:
            pass #wait several seconds before start uncursing
        
        if i == 1:
            for m in self.leg_meats:
                m.norm_dist_koef = 0.5
                
        if i == 2:        
            for a in self.eye_anim:
                a.delays[0] = 0
                if ((a.cur == 3) and (a.playing)):
                    a.stop()
        if i == 3:
            self.head.uncurse()
            for m in self.leg_meats:
                m.norm_dist_koef = 1.35
            for a in self.eye_anim:
                a.delays[0] = a.max_pause
                a.start() #we could a.start(3), but I like it better if
                          #eyes are immediately opened

        if i == 4:
           pass #wait till legs are spread widely 

        if i == 5:
            for m in self.leg_meats:
                m.norm_dist_koef = 1.0
            
                    
        

        
        
if __name__ == "__main__":
    x, y = (1, 2)
    x0, y0 = (1, 1)
    ang = angle_a_to_b_rad((x0, y0), (x, y))
    print ang
    r = dist_a_to_b((x0, y0), (x, y))
    newx, newy = x0 + r * cos(ang), y0 - r * sin(ang)
    print "\nold: (%0.3f, %0.3f)" % (x, y)
    print "new: (%0.3f, %0.3f)" % (newx, newy)
